=begin
------------------------------------------------------------------------
Copyright 2007-2017, TIG (c)
All rights reserved.
This software is provided "as is" and without any express or implied 
warranties, including, without limitation, the implied warranties of 
merchantability and fitness for a particular purpose.
------------------------------------------------------------------------
Name        : Roof.rb
Author      : TIG
Description : A set of Roof Making Tools.
Menu Item   : Plugins|Extensions > Roof... > Sub-Menus...
Usage       : See RoofHelp.mht/.pdf file originally distributed with this.
Version     : 
1.0	20070824	First issue.
1.1	20070826	A:B slope glitch fixed: array.to_f !
1.2	20070827	Rectangle checking OK =~ 90 degrees.
1.3	20070828	Fixed Gable Materials messed up by v1.2,
							gable 2 overhang glitches & spurious
							non-rectanglur face errors.
2.0	20070902	Dialogs redone to avoid Mac failures.
							Gable and Pyramid defs redone to picking
							3 points, rather than face/edge.
							Help redone.
2.1	20070903	Gable fascia material glitch fixed.
							Pyramid making made more efficient.
3.0	20071223	Sloping fascia and soffit options added.
							Additional warnings for odd geometry.
							Help files updated.
3.1 20110304	Unscrambled to avoid inexplicable clash 
							with 'Artisan' menu loading eval code.
							Rare reversed face glitches fixed.
							Changed typename to class, to speed tests.
3.2 20110331	Fixed reversed face glitches in fascias if 
							Hip's 'selected-face' is itself reversed.
							The offset_eaves code rewritten to correct 
							inexplicable irregularities in soffit sizes 
							in v3.1.  Listed materials' now use their 
							'display_name' rather than their 'name'.
3.3 20110331	Continuing issue with offset_eaves and size
							of soffits addressed.
3.4 20110403	Minor typo glitch with A:B pitches fixed.
3.5 20110612	@fstype="Vertical+Horizontal" fixed, also 
							issue with some distorted faces addressed, 
							Roof-Slope now reports to 3 d.p.
3.6 20110918	Errors minimized. Speed greatly improved.
							Slope reported in angle, 1:B and A:12.
3.7 20110926	Error/Glitch in Verge by Width resolved.               
4.0 20170121	Minor tweaks and v2017 compatibility fixes.
5.0 20171219  Typo in Gable ^Height corrected. Resigned for v2018.

------------------------------------------------------------------------
Some parts were inspired in part by some of Didier Burr's work and also 
RickW's...  
The first few methods are based on RickW's ideas...
[adjusted by TIG = ###]
=end
###---------------------------------------------------------------------
require 'sketchup.rb'
###---------------------------------------------------------------------
class Geom::Vector3d
 unless self.method_defined?(:to_flat_angle)
	 def to_flat_angle() ###
		vector=Geom::Vector3d.new(self.x,self.y,0)
		if vector.y < 0
			return 2*180.degrees-vector.angle_between(Geom::Vector3d.new(1,0,0))
		elsif vector.y > 0
			return vector.angle_between(Geom::Vector3d.new(1,0,0))
		elsif vector.y == 0
			if vector.samedirection?(Geom::Vector3d.new(1,0,0))
				return 0
			else
				return 180.degrees
			end#if
		end#if
	 end#def
 end
end#class
###---------------------------------------------------------------------
class Geom::Point3d
	unless self.method_defined?(:is_on_face?)
		def is_on_face?(face, bounds) ###
			return nil if not face.class==Sketchup::Face###
			origin=face.vertices[0].position
			x,y,z=face.normal.axes
			tr=Geom::Transformation.axes(origin,x,y,z)
					tr.invert!
			pt=self.clone
					pt.transform!(tr)
					pts=face.outer_loop.vertices.collect{|v|(v.position.transform(tr)).to_a}
			return true if Geom.point_in_polygon_2D(pt, pts, bounds) && pt.z==0
			return false
		end#def
	end
end#class
###---------------------------------------------------------------------
class Sketchup::Face
 unless self.method_defined?(:offset_eaves)
	 def offset_eaves(dist)### v3.3 rewritten
		#return nil if dist==0
		return nil if (not ((dist.class==Fixnum || dist.class==Float || dist.class==Length) && dist!=0))
		verts=self.outer_loop.vertices
		pts=[]
		0.upto(verts.length-1){|a|
			vec1=(verts[a].position-verts[a-(verts.length-1)].position).normalize
			vec2=(verts[a].position-verts[a-1].position).normalize
			vec3=(vec1+vec2).normalize
			if vec3.valid?
				ang=vec1.angle_between(vec2)/2
				ang=90.degrees if vec1.parallel?(vec2)
				vec3.length=dist/Math::sin(ang)
				t=Geom::Transformation.new(vec3)
				if pts.length > 0
					if not (vec2.parallel?(pts.last.vector_to(verts[a].position.transform(t))))
						t=Geom::Transformation.new(vec3.reverse)
					end
				end
				pts << (verts[a].position.transform(t))
			end
		}
		self.parent.entities.add_face(pts)
	 end#def
 end
end#class
### USED! v3.4
### This to fix [].to_f glitch ###--------------------------------------
class Array
unless self.method_defined?(:to_f)
		def to_f
			self.collect!{|e|e.to_f}; self
		end#def
	end
	unless self.method_defined?(:to_i)
		def to_i
			self.collect!{|e|e.to_i}; self
		end#def
	end
end#class Array
###--- end of RickW inspired stuff ###----------------------------------
class Float
	unless self.method_defined?(:is_near_to?)
		def is_near_to?(num2, tol=1000)### level of tolerance
			num1=(self*tol).to_i
			num2=(num2*tol).to_i
			return true if num1==num2
			return nil
		end
	end
end#class Float
class Length
	unless self.method_defined?(:is_near_to?)
		def is_near_to?(num2, tol=1000)### level of tolerance
			num1=(self*tol).to_i
			num2=(num2*tol).to_i
			return true if num1==num2
			return nil
		end
	end
end#class Length
###
class Roof ##############################################################
 ###
 def Roof::ordered_edge_points(face)
  ### a bit RickW inspired ?
  face.outer_loop.edges.each{|edge|
    face.parent.entities.add_line(edge.end.position,edge.start.position) if edge.reversed_in?(face)
  }
  verts=face.outer_loop.vertices
  spoints=[]
  verts.each{|v|spoints << v.position}
  ### ensure c/clockwise
  pt0=spoints[0]; pt1=spoints[1]
  pt0=Geom::Point3d.new(pt0[0],pt0[1],pt0[2]) 
  pt1=Geom::Point3d.new(pt1[0],pt1[1],pt1[2]) 
  vector=pt0.vector_to(pt1); pt2=pt0.offset(vector,0.1.mm)
  rotate=Geom::Transformation.rotation(pt0,Z_AXIS,0.1.degrees)
  pt2=pt2.transform!(rotate)
  spoints.reverse! if not pt2.is_on_face?(face,true)
  ###
  return spoints
end#def
###---------------------------------------------------------------------


def Roof::hip()
  ### version error ###-------------------------------------------------
  if Sketchup.version.to_i<5
     UI.messagebox("Sorry but 'Hipped Roof' only works with Version 5 and over...")
     return nil
  end#if
  ### check face -------------------------------------------------------
  @model=Sketchup.active_model
  ss=@model.selection
  faces=[]
  ss.each{|e|faces << e if e.class==Sketchup::Face}
  if ss.empty? || faces.empty?
    UI.messagebox("Hipped Roof:\n\nYou must select one horizontal FACE !")
    return nil
  end#if
  if faces[1]
    UI.messagebox("Hipped Roof:\n\nYou must select ONE horizontal face !")
    return nil
  end#if
  oface=faces[0]### starting face
  if oface.normal.z.abs !=1.0
    UI.messagebox("Hipped Roof:\n\nYou must select one HORIZONTAL face !")
    return nil
  end#if
def Roof::dialog_hip ()
### hip dialog ###----------------------------------------------------------
@pitch="1:2" if ! @pitch && @model.options["UnitsOptions"]["LengthUnit"]<2
@pitch="30" if ! @pitch && @model.options["UnitsOptions"]["LengthUnit"]>1
@fascia=8.inch if ! @fascia && @model.options["UnitsOptions"]["LengthUnit"]<2
@fascia=200.mm if ! @fascia && @model.options["UnitsOptions"]["LengthUnit"]>1
@soffit=16.inch if ! @soffit && @model.options["UnitsOptions"]["LengthUnit"]<2
@soffit=400.mm if ! @soffit && @model.options["UnitsOptions"]["LengthUnit"]>1
@fstype="Vertical+Horizontal" if ! @fstype###v3.0
fstypes="Vertical+Horizontal|Vertical+Sloping|Sloping+Sloping|Sloping+Horizontal"###v3.0
@other="No" if ! @other
units="inches" if @model.options["UnitsOptions"]["LengthUnit"]==0
units="feet" if @model.options["UnitsOptions"]["LengthUnit"]==1
units="mm" if @model.options["UnitsOptions"]["LengthUnit"]==2
units="cm" if @model.options["UnitsOptions"]["LengthUnit"]==3
units="m" if @model.options["UnitsOptions"]["LengthUnit"]==4
enums=["","","",fstypes,"Yes|No"]
prompts=["Roof Slope (Degrees OR A:B):  ","Eaves Fascia Size (#{units}):  ","Eaves Soffit Size (#{units}):  ","Fascia+Soffit Type:  ","Set Layer and Materials ? :"]###v3.0
values=[@pitch,@fascia,@soffit,@fstype,@other]###v3.0
@results=(inputbox prompts,values,enums,"Hipped Roof Settings")
return nil if ! @results
blank=false###
@results.each{|e|blank=true if e.to_s==""}
if blank
  UI.messagebox("Hipped Roof:\n\nYou cannot have blank entries.\nRetry...")
  Roof::dialog_hip
end#if###
(@pitch_s,@fascia_,@soffit_,@fstype_,@other_)=@results###v3.0
@pitch=@pitch_s;@fascia=@fascia_;@soffit=@soffit_
@fstype=@fstype_;@other=@other_###v3.0
### get angle of roof in radians
ratio=@pitch_s.split(":").to_f ### v3.4
if ratio[1]
  tan=ratio[0]/ratio[1]
  @pitch_d=(Math::atan(tan)).radians
else
  @pitch_d=ratio[0]
end#if
@pitch_r=@pitch_d.degrees  ### to radians ###------------------------
if @pitch_d<1
  UI.messagebox("Hipped Roof:\n\nPitch can't be less than 1 degree.")
  @pitch="1";@pitch_s="1";@pitch_d=1;@pitch_r=1.degrees
  Roof::dialog_hip
end#if
if @pitch_d>89
  UI.messagebox("Hipped Roof:\n\nPitch can't be more than 89 degrees.")
  @pitch="89";@pitch_s="89";@pitch_d=89;@pitch_r=89.degrees
  Roof::dialog_hip
end#if
if @soffit_ <0
  UI.messagebox("Hipped Roof:\n\nSoffit can't be LESS THAN ZERO.\nResetting it to ZERO...\n\nFor a 'parapet-eaves' first use 'Offset' on the face to inset its edges ...")
  @soffit=0.to_l;@soffit_=0.to_l
  Roof::dialog_hip
end#if
###v3.0
if @fascia_ <0
  UI.messagebox("Hipped Roof:\n\Fascia can't be LESS THAN ZERO.\nResetting it to ZERO...")
  @fascia=0.to_l;@fascia_=0.to_l
  Roof::dialog_hip
end#if
if @fascia_ ==0 && @fstype_=="Vertical+Sloping"
  UI.messagebox("Hipped Roof:\n\nIf the Fascia is ZERO then you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
  @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
  Roof::dialog_hip
end#if
if @fascia_ ==0 && @fstype_=="Sloping+Sloping"
  UI.messagebox("Hipped Roof:\n\nIf the Fascia is ZERO then you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
  @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
  Roof::dialog_hip
end#if
###v3.0
###
end#def dialog
### run hip dialog
  Roof::dialog_hip()
  return nil if ! @results
###
###v3.0 sort out sloping fascia and soffit
### defaults=Vertical+Horizontal
fac=@fascia; fac_up=fac; fac_out=0.0
sof=@soffit; sof_up=0.0
if @fstype=~/Sloping\+/
  fac_up =fac*Math::cos(@pitch_r)
  fac_out=fac*Math::sin(@pitch_r)
end#if
if @fstype=~/Sloping$/
  sof_up=sof*Math::tan(@pitch_r)
end#if
###v3.0
### do hipped roof ###--------------------------------------------------
@model.start_operation("Hipped Roof")
entities=@model.active_entities
  ###
  tpts=Roof.ordered_edge_points(oface)
  oface=entities.add_face(tpts)
  ###
oedges=oface.outer_loop.edges
overts=oface.outer_loop.vertices
if oface.normal.z<0 ### v3.2
  oedges.reverse!
  overts.reverse!
end#if
count=overts.length### used for % etc later...
opoints=[];overts.each{|e|next if not e.valid?; opoints << e.position.to_a}
### set up roof group etc ###-------------------------------------------
roof_group=entities.add_group
roof_entities=roof_group.entities
iface=roof_entities.add_face(opoints)
if sof==0###v3.0
  face=iface
else
  face=iface.offset_eaves(sof+fac_out)###v3.0
  if not face
    face=iface
  end#if
end#if
### Trap for inverted roof offsets
perimi=0; iface.outer_loop.edges.each{|e|perimi+=e.length}
perimo=0;  face.outer_loop.edges.each{|e|perimo+=e.length}
if perimo < perimi
  face=iface.offset_eaves(-sof-fac_out)
  sof= -sof
  fac_out= -fac_out
  if not face
    face=iface
  end#if
end
###
###v3.0 roof now consists of 2 loops and 1 face
### trap for followme bugsplat if face has corner at origin ###---------
origin=false
if roof_group.bounds.min.z==0
  origin=true
  orpt=roof_group.bounds.min
  pt0=Geom::Point3d.new(0,0,1+(fac*fac))
  tr0=Geom::Transformation.new(pt0)
  roof_group.move!(tr0)
end#if
### --------------------------------------------------------------------
### continue with roof group set up ###---------------------------------
eedges=face.outer_loop.edges
everts=face.outer_loop.vertices###
epoints=[];everts.each{|e|epoints << e.position.to_a}
### remove any inner loop edges in roof group...
edges_in=[];edges_out=[]
roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Edge
    if e.faces[1]### inner loop
      edges_in << e
    else### outer loop
      edges_out << e
    end#if
  end#if
}
edges_in.each{|e|e.erase!} if sof > 0
#edges_out.each{|e|e.erase!} if sof < 0
face=edges_in[0].faces[0] if edges_in[0] && edges_in[0].valid?
face=edges_out[0].faces[0] if edges_out[0] && edges_out[0].valid?
epoints=Roof.ordered_edge_points(face)
eedges=face.outer_loop.edges
eedges.each{|e|e.erase! if e.valid?}
face=roof_entities.add_face(epoints)
eedges=face.outer_loop.edges
everts=face.outer_loop.vertices
### make path and profile etc ###---------------------------------------
### work out which vertex gives largest dist dim ###--------------------
dims=[0.mm,nil]###for dist,evert
down=1
everts.each{|evert|
  Sketchup::set_status_text("Hipped Roof: Finding Faces "+(100*down/count).to_i.to_s+"%. Please Wait...",SB_PROMPT)
  down=down+1
  dist=1.mm ### = min roof size !
  edge1orig=evert
  edge2orig=evert
  edge1=edge1orig.edges[0]
  edge2=edge2orig.edges[1]
  edge1other=edge1.other_vertex(edge1orig)
  edge2other=edge2.other_vertex(edge2orig)
  vector1=edge1orig.position.vector_to(edge1other.position)
  vector2=edge2orig.position.vector_to(edge2other.position)
  angle1=vector1.to_flat_angle
  angle2=vector2.to_flat_angle
  angle_between=angle1-angle2
  angle=angle_between/2
  pt1=evert.position
  pt2=pt1.offset(vector2,1.mm)
  rotate=Geom::Transformation.rotation(pt1,[0,0,1],angle)
  pt2=pt2.transform!(rotate)
  ### sort which angle gives a point on the face ###
  if angle > 0
    angle=angle+180.degrees if not pt2.is_on_face?(face,true)
  else
    angle=angle-180.degrees if not pt2.is_on_face?(face,true)
  end#if
  ### find biggest one
  on_face=true
  while on_face
    pt2=pt1.offset(vector2,dist)
    rotate=Geom::Transformation.rotation(pt1,[0,0,1],angle)
    pt2=pt2.transform!(rotate)
    if not pt2.is_on_face?(face,true) ### gone off the face
      if dist > dims[0]
        dims=[dist-100.mm,evert]
      end#if
      on_face=false
    end#if
    dist=dist+100.mm###increment dist 100mm till it goes off face.
  end#while
}#end everts.each
### largest dist ###----------------------------------------------------
dist=(dims[0]/1.01).abs#.1570054### never > (sqrt3 - (1/sqrt3)) ? ###
evert=dims[1]
  edge1orig=evert
  edge2orig=evert
  edge1=edge1orig.edges[0]
  edge2=edge2orig.edges[1]
  edge1other=edge1.other_vertex(edge1orig)
  edge2other=edge2.other_vertex(edge2orig)
  vector1=edge1orig.position.vector_to(edge1other.position)
  vector2=edge2orig.position.vector_to(edge2other.position)
  angle1=vector1.to_flat_angle
  angle2=vector2.to_flat_angle
  angle_between=angle1-angle2
  angle=angle_between/2
  pt1=evert.position
  pt2=pt1.offset(vector2,1.mm)
  rotate=Geom::Transformation.rotation(pt1,[0,0,1],angle)
  pt2=pt2.transform!(rotate)
  ### sort which angle gives a point on the face ###
  if angle > 0
    angle=angle+180.degrees if not pt2.is_on_face?(face,true)
  else
    angle=angle-180.degrees if not pt2.is_on_face?(face,true)
  end#if
  pt2=pt1.offset(vector2,dist)
  rotate=Geom::Transformation.rotation(pt1,[0,0,1],angle)
  pt2=pt2.transform!(rotate)
### sort out transformation ###-----------------------------------------
pt1=Geom::Point3d.new(pt1).transform(roof_group.transformation)if not origin
pt2=Geom::Point3d.new(pt2).transform(roof_group.transformation)if not origin
### need to adjust opp for being on diagonal (sin)
fangle=angle.abs
if fangle >0
 while fangle > 180.degrees
  fangle=fangle-180.degrees
 end#while
end#if
###
### get opposite = height of ridge at dist from pt1
opp=(dist*Math::sin(fangle)*Math::tan(@pitch_r)).abs
### make upside down triangular face for followme ###-------------------
pta=pt1
ptb=[pt1.x,pt1.y,pt1.z+opp]
ptc=[pt2.x,pt2.y,pt2.z+opp]
fface=roof_entities.add_face(pta,ptb,ptc)
### re-sort so starts from pt1
index=0
0.upto(eedges.length-1) do |i|
  index=i if eedges[i].end.position==pt1
end#upto i
tedges=(eedges.slice(index..-1))+(eedges.slice(0..index-1))
###
fface.followme(tedges)### make main roof faces ###---------------------
### The tidy up ###-----------------------------------------------------
### Intersect :
### The intersect_with method is used to intersect entities, a component
### instance, or group ### with an entities object.
###   entities.intersect_with(recurse,transformation1,entities1,transformation2,hidden,entities2)
### recurse - true if you want this entities object to be recursed .
###   (intersection lines will be put inside of groups and components 
###    within this entities object)
### transformation1 - the transformation for this entities object.
### entities1 - the entities (group etc) where you want the intersection
### lines to appear.
### transformation2 - the transformation for entities1.
### hidden - true if you want hidden geometry in this entities-object to 
### be used in the intersection.
### entities2 - an entities object (or an array of entities ?) = 'faces'.
Sketchup::set_status_text("Hipped: Finding Edges. Please Wait...",SB_PROMPT)
rtr=Geom::Transformation.new()
count.times{
  roof_entities.intersect_with(true,rtr,roof_entities,rtr,true,roof_entities.to_a)
}
### rebuild faces
roof_entities.to_a.each{|e|
  next unless e.valid?
  next unless e.class==Sketchup::Face
  e.reverse! if e.normal.z<0
  nn=e.normal
  eds=e.edges
  e.erase!
  eds.each{|ee|
    next unless ee.valid?
    ee.find_faces
    ee.faces.each{|f|f.reverse! if f.normal==nn.reverse}
  }
}
#UI.messagebox('start')
#return
### It repeats to catch all intersections in complex sets---------------
Sketchup::set_status_text("Hipped Roof: Finishing.",SB_PROMPT)
zz=roof_group.bounds.min.z ### needed later...
rep=(count*count/2).to_i
### remove flat faces...
#rep.times{
 roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Face && e != face
    e.erase! if e.normal.x.abs<1.0e-006 && e.normal.y.abs<1.0e-006
  end#if
 }
#}
Sketchup::set_status_text("Hipped Roof: Finishing..",SB_PROMPT)
### erase all "vertical" edges...
#rep.times{
 roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Edge
    e.erase! if e.start.position.x==e.end.position.x && e.start.position.y==e.end.position.y
  end#if
 }
#}
Sketchup::set_status_text("Hipped Roof: Finishing...",SB_PROMPT)
### erase faceless edges...
#rep.times{
 roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Edge
    e.erase! if ! e.faces[0]
  end#if
 }
#}
Sketchup::set_status_text("Hipped Roof: Finishing....",SB_PROMPT)
### remove one face edges = off base...
#rep.times{
 roof_entities.to_a.each{|e|
  #Sketchup.active_model.active_view.refresh
  if e.valid? and e.class==Sketchup::Edge and e.bounds.min.z!=zz
    e.erase! if ! e.faces[1]
  end#if
 }
#}
Sketchup::set_status_text("Hipped Roof: Finishing.....",SB_PROMPT)
### now reverse faces so 'up'
roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Face
    e.reverse! if e.normal.z <0
  end#if
}
Sketchup::set_status_text("Hipped Roof: Finishing......",SB_PROMPT)
### remove all 'flat' faces again...
#rep.times{
 roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Face
    e.erase! if e.normal.x.abs<1.0e-006 && e.normal.y.abs<1.0e-006
  end#if
 }
#}
Sketchup::set_status_text("Hipped Roof: Finishing.......",SB_PROMPT)
### trap any non-base non-flat faces...
faces_hi=[]
roof_entities.to_a.each{|e|
  attached=true
  if e.valid? and e.class==Sketchup::Face && e.bounds.min.z > zz
    e.edges.each{|ee|
      zzz=e.bounds.min.z ### bottom edges
      if ee.valid? and ee.bounds.min.z==zzz && ee.start.position.z==ee.end.position.z ### it is level
        otherfaces=ee.faces-[e]
        otherfaces.each{|f|
          attached=false if f.bounds.min.z != zz
        }
      end#if
    }
    faces_hi << e if not attached
  end#if
}
faces_hi.each{|f|
  coplanar=false
  f.edges.each{|e|
    coplanar=true if e.faces[1] && e.faces[0].normal==e.faces[1].normal
    coplanar=true if e.faces[2] && e.faces[0].normal==e.faces[2].normal
    coplanar=true if e.faces[2] && e.faces[1].normal==e.faces[2].normal
    coplanar=true if e.faces[3] && e.faces[0].normal==e.faces[3].normal
    coplanar=true if e.faces[3] && e.faces[1].normal==e.faces[3].normal
    coplanar=true if e.faces[3] && e.faces[2].normal==e.faces[3].normal
  }
  f.erase! if f.valid? && ! coplanar
}
faces_hi.each{|f|
  if f.valid?
    f.edges.each{|e|
      e.erase! if e.valid? && ! e.faces[1]
    }
  end#if
}
Sketchup::set_status_text("Hipped Roof: Finishing.......",SB_PROMPT)
### tidy up faceless and coplanar edges etc...
#rep.times{
 roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Edge
    e.erase! if ! e.faces[0]### faceless
  end#if 
  if e.valid? && e.class==Sketchup::Edge
    e.erase! if e.faces[1] && e.faces[0].normal==e.faces[1].normal###coplanar
  end#if
 }
#}
Sketchup::set_status_text("Hipped Roof: Finishing........",SB_PROMPT)
### remove one face edges that are above base and only have one face...
#rep.times{
 roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Face
    if e.bounds.min.z > zz
      edges=e.edges
      edges.each{|ee|ee.erase! if ee.valid? && ! ee.faces[1]}
    end#if
  end#if
 }
#}
Sketchup::set_status_text("Hipped Roof: Finishing.........",SB_PROMPT)
### erase any 'floating' bits of roof above the main body...
#rep.times{
 roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Face && faces_hi.include?(e)
    if e.bounds.min.z > zz
      edges=e.edges
      edges.each{|ee|ee.erase! if ee.valid?}
    end#if
  end#if
 }
#}
Sketchup::set_status_text("Hipped Roof: Finished.",SB_PROMPT)
### erase faceless edges again...
#rep.times{
 roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Edge
    e.erase! if ! e.faces[0]
  end#if
 }
#}
### trap for failure
broken=false
roof_entities.each{|e|
  if e.class==Sketchup::Edge and e.line[1].z.abs>=0.001 and not e.faces[1]
    broken=true
    break
  end#if
}
roof_entities.erase_entities(roof_entities.to_a) if broken
### if fails...
if ! roof_entities[0] || roof_entities.length <= count*2
  UI.messagebox("Hipped Roof Error:\n\nIt was not possible to make a single pitch hipped roof from this face.\n\nSometimes the face/edge relationships get muddled in the model's database.\nTry re-making the original face:\n- Delete it and redraw over one edge to remake it.  \nOr copy the face to one side, delete the original and move the copy back.  \nThis methods can often make a face 'roof'.\n\nIf not, then try sub-dividing the face into pieces and joining their roofs manually.\n\nNote that some face shapes can never be roofed with a single pitch.\ne.g. crooked stars.\n\nExiting...")
  roof_group.erase! if roof_group.valid?
  ### tidy up at end ###--------------------------------------------------
  @model.selection.clear
  @model.selection.add(oface)### end with original face re-selected
  @model.active_view.invalidate
  @model.commit_operation
  Sketchup.send_action"selectSelectionTool:"
  return nil
end#if
###---------------------------------------------------------------------
### correct for fascia
roof_group.move!(Geom::Transformation.new([0,0,fac_up]))if fac_up>0
###
### sort out soffit ###-------------------------------------------------
soffit_group=entities.add_group###v3.0
soffit_entities=soffit_group.entities###v3.0
face=soffit_entities.add_face(opoints)###v3.0
if sof!=0 ###v3.0
  sface=face.offset_eaves(sof)
  if not sface
    sface=face
  end#if
  verts=face.vertices
  sverts=sface.outer_loop.vertices
  sface.reverse! if sface.normal.z>0
end#if
### Trap for inverted roof offsets
perimi=0; face.outer_loop.edges.each{|e|perimi+=e.length}
perimo=0; sface.outer_loop.edges.each{|e|perimo+=e.length}
if perimo < perimi
  sface=face.offset_eaves(-sof-fac_out)
  sof= -sof
  fac_out= -fac_out
  if not sface
    sface=face
  end#if
end
face.erase! if face.valid?### make the hole
###
if sof!=0 && sof_up>0
  ### fold soffit if sloping...
 verts.each{|v|
  next if not v.valid?
  pos=v.position
  posn=[pos.x,pos.y,pos.z+sof_up]
  vec=pos.vector_to(posn)
  tran=Geom::Transformation.translation(vec)
  soffit_entities.transform_entities(tran,v)
 }
 ### erase coplanar edges.
 soffit_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Edge
    if e.faces[1] and not e.faces[2]
	  e.erase! if e.faces[0].normal==e.faces[1].normal and e.valid?
	end#if
	e.erase! if e.valid? and not e.faces[0]
  end#if
 }###
end#if sof_up
### move whole roof group down by sof_up so it sits on selected face.
trz=Geom::Transformation.new([0,0,sof_up*-1+fac_up])if sof_up>0###v3.0
roof_group.move!(trz)if sof_up>0###v3.0
trz=Geom::Transformation.new([0,0,sof_up*-1])if sof_up>0###v3.0
soffit_group.move!(trz)if sof_up>0###v3.0
###v3.0
### sort out fascia etc ###----------------------------------------------
fascia_group=entities.add_group
fascia_entities=fascia_group.entities
face=fascia_entities.add_face(opoints)
###
if fac_out==0
  fface=face.offset_eaves(sof) if sof!=0
  fface=face if sof==0
  xverts=fface.outer_loop.vertices
  xxpoints=[]
  xverts.each{|e|
    next if not e.valid?
    ee=e.position.to_a
    xxpoints << [ee[0],ee[1],ee[2]+fac_up]
  }
  edges2go=face.edges if sof!=0
  edges2go.each{|e|e.erase!} if sof!=0
  fascia_entities.add_face(xxpoints)
  fascia_entities.to_a.each{|e|e.erase! if e.valid? and e.class==Sketchup::Face}
  vlines=[]
  xverts.each{|e|
    next if not e.valid?
    ee=e.position.to_a
    v=fascia_entities.add_line(ee,[ee[0],ee[1],ee[2]+fac_up])
    vlines << v
  }
  vlines.each{|e|e.find_faces if e}
  fascia_entities.to_a.each{|e|e.reverse! if e.valid? and e.class==Sketchup::Face and oface.bounds.min.z!=0}
else ### fac_out > 0 ==sloping
  fface=face.offset_eaves(sof) if sof!=0
  fface=face if sof==0
  edges2go=face.edges if sof!=0
  ffface=fface.offset_eaves(fac_out)
  ffface.reverse! if ffface.normal.z>0
  fface.erase! if fface.valid?
  face.erase! if face.valid?
  edges2go.each{|e|e.erase! if e.valid?} if sof!=0
  fverts=ffface.outer_loop.vertices
  fverts.each{|v|
    next if not v.valid?
    pos=v.position
    posn=[pos.x,pos.y,pos.z+fac_up]
    vec=pos.vector_to(posn)
    tran=Geom::Transformation.translation(vec)
    fascia_entities.transform_entities(tran,v)
  }
  ### erase coplanar edges.
  fascia_entities.to_a.each{|e|
    if e.valid? and e.class==Sketchup::Edge
      if e.faces[1] and not e.faces[2]
	    e.erase! if e.faces[0].normal==e.faces[1].normal and e.valid?
	  end#if
	  e.erase! if e.valid? and not e.faces[0]
    end#if
  }
end#if
### move whole fascia group down by sof_up so it sits on selected face.
trz=Geom::Transformation.new([0,0,sof_up*-1])
fascia_group.move!(trz)
### move origin based faces etc back into place to avoid bugsplats ###--
if origin
  tr=Geom::Transformation.new(orpt)
  trz=Geom::Transformation.new([0,0,0])
  roof_group.move!(trz)
  ### correct for fascia ?
  roof_group.move!(Geom::Transformation.new([0,0,fac_up]))if fac_up>0
  soffit_group.move!(trz)
  fascia_group.move!(trz)
end#if
###v3.0
### Moved materials dialog to after roof is made...
def Roof::dialog_mats_hip ()
### materials dialog ###------------------------------------------------
  model=Sketchup.active_model
  ### layers...
  mlayers=model.layers
  alayer=model.active_layer
  model.active_layer=nil
  deflayer=model.active_layer.name
  model.active_layer=alayer
  rlayer="-ROOF"### edit this to suit your desired default roof layer.
  layers=[]
  mlayers.each{|layer|layers << layer.name if layer.name!=deflayer}
  layers=[rlayer,deflayer]+layers.sort!
  layers=layers.uniq.join('|')
  ### materials...
  mat_roof="Roof Surface"
  mat_fascia="Roof Fascia"
  mat_soffit="Roof Soffit"
  materials=model.materials
  mats=[]
  materials.each{|mat|mats << mat.display_name}
  mats.sort!
  list_mat_roof=((["<Default>"]+[mat_roof]+mats).uniq).join("|")
  list_mat_fascia=((["<Default>"]+[mat_fascia]+mats).uniq).join("|")
  list_mat_soffit=((["<Default>"]+[mat_soffit]+mats).uniq).join("|")
  ### set up variables...
  @layer=rlayer if ! @layer
  @mat_roof=mat_roof if ! @mat_roof
  @mat_fascia=mat_fascia if ! @mat_fascia
  @mat_soffit=mat_soffit if ! @mat_soffit
  enums=[layers,list_mat_roof,list_mat_fascia,list_mat_soffit]
  prompts=["Roof Layer:  ","Roof Surface:  ","Roof Fascia:  ","Roof Soffit: "]
  values=[@layer,@mat_roof,@mat_fascia,@mat_soffit]
  @other_results=(inputbox prompts,values,enums,"Roof Layer and Materials")
  @layer,@mat_roof,@mat_fascia,@mat_soffit=@other_results if @other_results
  @other="No" if ! @other_results
  ###
end#def
### run material dialog
  Roof::dialog_mats_hip() if @other=="Yes"
### 
###v3.0
### sort out layer, materials etc ###-----------------------------------
if @other=="Yes"
  ### note that layer is set after groups combined...
  extg=nil;@model.layers.each{|lyr|extg=lyr if lyr.name==@layer}
  if @layer=="<Default>"
    layr=nil
  else
    if extg
      layr=extg
    else
      layr=@model.layers.add(@layer)
    end#if
  end#if
  ### materials...
  ### roof
  extg=nil;@model.materials.each{|mat|extg=mat if mat.display_name==@mat_roof}
  if @mat_roof=="<Default>"
    mat_r=nil
  else
    if extg
      mat_r=extg
    else
      mat_r=@model.materials.add(@mat_roof)
      mat_r.color=[111,111,111]### adjust default colours here.
    end#if
  end#if
  roof_entities.to_a.each{|e|e.material=mat_r if e.valid? and e.class==Sketchup::Face}
  ### fascia
  extg=nil;@model.materials.each{|mat|extg=mat if mat.display_name==@mat_fascia}
  if @mat_fascia=="<Default>"
    mat_f=nil
  else
    if extg
      mat_f=extg
    else
      mat_f=@model.materials.add(@mat_fascia)
      mat_f.color=[66,66,66]### adjust default colours here.
    end#if
  end#if
  fascia_entities.to_a.each{|e|e.material=mat_f if e.valid? and e.class==Sketchup::Face}
  ### soffit
  extg=nil;@model.materials.each{|mat|extg=mat if mat.display_name==@mat_soffit}
  if @mat_soffit=="<Default>"
    mat_s=nil
  else
    if extg
      mat_s=extg
    else
      mat_s=@model.materials.add(@mat_soffit)
      mat_s.color=[222,222,222]### adjust default colours here.
    end#if
  end#if
  soffit_entities.to_a.each{|e|e.material=mat_s if e.valid? and e.class==Sketchup::Face}
  ###
end#if
### combine pieces ###--------------------------------------------------
tgroup=entities.add_group(roof_group,soffit_group,fascia_group)
roof_group.explode
soffit_group.explode
fascia_group.explode
roof_group=tgroup
###
roof_group.layer=layr if @other=="Yes"### layer set here...
### tidy up at end ###--------------------------------------------------
@model.selection.clear
@model.selection.add(oface)### end with original face re-selected
@model.active_view.invalidate
@model.commit_operation
Sketchup.send_action"selectSelectionTool:"
  ###
end#def hip
###---------------------------------------------------------------------



def Roof::mansard()
  ### version error ###-------------------------------------------------
  if Sketchup.version.to_i<5
     UI.messagebox("Sorry but 'Mansard/Sprocket Roof' only works with Version 5 and over...")
     return nil
  end#if
  ### check face -------------------------------------------------------
  @model=Sketchup.active_model
  ss=@model.selection
  faces=[]
  ss.each{|e|faces << e if e.class==Sketchup::Face}
  if ss.empty? || faces.empty?
    UI.messagebox("Mansard/Sprocket Roof:\n\nYou must select one horizontal FACE !")
    return nil
  end#if
  if faces[1]
    UI.messagebox("Mansard/Sprocket Roof:\n\nYou must select ONE horizontal face !")
    return nil
  end#if
  oface=faces[0]### starting face
  if oface.normal.z.abs !=1.0
    UI.messagebox("Mansard/Sprocket Roof:\n\nYou must select one HORIZONTAL face !")
    return nil
  end#if
def Roof::dialog_man ()
### man dialog ###----------------------------------------------------------
@pitch1="2:1" if ! @pitch1 && @model.options["UnitsOptions"]["LengthUnit"]<2
@pitch1="60" if ! @pitch1 && @model.options["UnitsOptions"]["LengthUnit"]>1
@pitch2="1:2" if ! @pitch2 && @model.options["UnitsOptions"]["LengthUnit"]<2
@pitch2="30" if ! @pitch2 && @model.options["UnitsOptions"]["LengthUnit"]>1
@ht=48.inch if ! @ht && @model.options["UnitsOptions"]["LengthUnit"]<2
@ht=1200.mm if ! @ht && @model.options["UnitsOptions"]["LengthUnit"]>1
@fascia=8.inch if ! @fascia && @model.options["UnitsOptions"]["LengthUnit"]<2
@fascia=200.mm if ! @fascia && @model.options["UnitsOptions"]["LengthUnit"]>1
@soffit=16.inch if ! @soffit && @model.options["UnitsOptions"]["LengthUnit"]<2
@soffit=400.mm if ! @soffit && @model.options["UnitsOptions"]["LengthUnit"]>1
@fstype="Vertical+Horizontal" if ! @fstype###v3.0
fstypes="Vertical+Horizontal|Vertical+Sloping|Sloping+Sloping|Sloping+Horizontal"###v3.0
@other="No" if ! @other
units="inches" if @model.options["UnitsOptions"]["LengthUnit"]==0
units="feet" if @model.options["UnitsOptions"]["LengthUnit"]==1
units="mm" if @model.options["UnitsOptions"]["LengthUnit"]==2
units="cm" if @model.options["UnitsOptions"]["LengthUnit"]==3
units="m" if @model.options["UnitsOptions"]["LengthUnit"]==4
enums=["","","","","",fstypes,"Yes|No"]
prompts=["Lower Slope (Degrees OR A:B):  ","Upper Slope (Degrees OR A:B):  ","Height of Lower Roof (#{units}):  ","Eaves Fascia Size (#{units}):  ","Eaves Soffit Size (#{units}):  ","Fascia+Soffit Type:  ","Set Layer + Materials ? :"]###v3.0
values=[@pitch1,@pitch2,@ht,@fascia,@soffit,@fstype,@other]###v3.0
@results=(inputbox prompts,values,enums,"Mansard/Sprocket Roof Settings")
return nil if ! @results
blank=false###
@results.each{|e|blank=true if e.to_s==""}
if blank
  UI.messagebox("Mansard/Sprocket:\n\nYou cannot have blank entries.\nRetry...")
  Roof::dialog_man
end#if###
(@pitch1_s,@pitch2_s,@ht_,@fascia_,@soffit_,@fstype_,@other_)=@results###v3.0
@pitch1=@pitch1_s;@pitch2=@pitch2_s;@ht=@ht_;
@fascia=@fascia_;@soffit=@soffit_;@fstype=@fstype_;###v3.0
@other=@other_
### get angle of roof in radians
ratio=@pitch1_s.split(":").to_f
if ratio[1]
  tan=ratio[0]/ratio[1]
  @pitch1_d=(Math::atan(tan)).radians
else
  @pitch1_d=ratio[0]
end#if
@pitch1_r=@pitch1_d.degrees  ### to radians ###------------------------
ratio=@pitch2_s.split(":").to_f
if ratio[1]
  tan=ratio[0]/ratio[1]
  @pitch2_d=(Math::atan(tan)).radians
else
  @pitch2_d=ratio[0]
end#if
@pitch2_r=@pitch2_d.degrees  ### to radians ###------------------------
if @pitch1_d<1
  UI.messagebox("Mansard/Sprocket Roof:\n\nLower pitch can't be less than 1 degree.")
  @pitch1="1";@pitch1_s="1";@pitch1_d=1;@pitch1_r=1.degrees
  Roof::dialog_man
end#if
if @pitch2_d<1
  UI.messagebox("Mansard/Sprocket Roof:\n\nUpper pitch can't be less than 1 degree.")
  @pitch2="1";@pitch2_s="1";@pitch2_d=1;@pitch2_r=1.degrees
  Roof::dialog_man
end#if
if @pitch1_d>89 
  UI.messagebox("Mansard/Sprocket Roof:\n\nLower pitch can't be more than 89 degrees.")
  @pitch1="89";@pitch1_s="89";@pitch1_d=89;@pitch1_r=89.degrees
  Roof::dialog_man
end#if
if @pitch2_d>89 
  UI.messagebox("Mansard/Sprocket Roof:\n\nUpper pitch can't be more than 89 degrees.")
  @pitch2="89";@pitch2_s="89";@pitch2_d=89;@pitch2_r=89.degrees
  Roof::dialog_man
end#if
if @soffit_ <0
  UI.messagebox("Mansard/Sprocket Roof:\n\nEaves can't be LESS THAN ZERO.\nResetting it to ZERO...\n\nFor a 'parapet-eaves' first use 'Offset' on the face to inset its edges ...")
  @soffit=0.to_l;@soffit_=0.to_l
  Roof::dialog_man
end#if
###v3.0
if @fascia_ <0
  UI.messagebox("Mansard/Sprocket Roof:\n\Fascia can't be LESS THAN ZERO.\nResetting it to ZERO...")
  @fascia=0.to_l;@fascia_=0.to_l
  Roof::dialog_man
end#if
if @fascia_ ==0 && @fstype_=="Vertical+Sloping"
  UI.messagebox("Mansard/Sprocket Roof:\n\nIf the Fascia is ZERO then you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
  @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
  Roof::dialog_man
end#if
if @fascia_ ==0 && @fstype_=="Sloping+Sloping"
  UI.messagebox("Mansard/Sprocket Roof:\n\nIf the Fascia is ZERO then you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
  @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
  Roof::dialog_man
end#if
###v3.0
###
if @ht_ <0
  UI.messagebox("Mansard/Sprocket Roof:\n\nLower Roof Height can't be LESS THAN ZERO.\n\nResetting it to ZERO...")
  @ht=0.to_l;@ht_=0.to_l
  Roof::dialog_man
end#if
###
end#def
### run man dialog
  Roof::dialog_man()
  return nil if ! @results
###
###v3.0 sort out sloping fascia and soffit
### defaults=Vertical+Horizontal
fac=@fascia; fac_up=fac; fac_out=0.0
sof=@soffit; sof_up=0.0
if @fstype=~/Sloping\+/
  fac_up =fac*Math::cos(@pitch1_r)
  fac_out=fac*Math::sin(@pitch1_r)
end#if
if @fstype=~/Sloping$/
  sof_up=sof*Math::tan(@pitch1_r)
end#if
###v3.0
### do Mansard/Sprocket roof ###--------------------------------------------------
@model.start_operation("Mansard/Sprocket Roof")
entities=@model.active_entities
  ###
  tpts=Roof.ordered_edge_points(oface)
  oface=entities.add_face(tpts)
  ###
oedges=oface.outer_loop.edges
overts=oface.outer_loop.vertices
if oface.normal.z<0 ### v3.2
  oedges.reverse!
  overts.reverse!
end#if
count=overts.length### used for % etc later...
opoints=[];overts.each{|e|next if not e.valid?; opoints << e.position.to_a}
### set up roof group etc ###-------------------------------------------
roof_group=entities.add_group
roof_entities=roof_group.entities
face=roof_entities.add_face(opoints)
if sof!=0 ###v3.0
  sface=face.offset_eaves(sof)
  if not sface
    sface=face
  end#if
  verts=face.vertices
  sverts=sface.outer_loop.vertices
  sface.reverse! if sface.normal.z>0
end#if
### Trap for inverted roof offsets
perimi=0; face.outer_loop.edges.each{|e|perimi+=e.length}
perimo=0; sface.outer_loop.edges.each{|e|perimo+=e.length}
if perimo < perimi
  sface=face.offset_eaves(-sof-fac_out)
  sof= -sof
  fac_out= -fac_out
  if not sface
    sface=face
  end#if
end
face.erase! if face.valid? ### make the hole
face=sface
###v3.0 roof now consists of 2 loops and 1 face
### trap for followme bugsplat if face has corner at origin ###---------
origin=false
if roof_group.bounds.min.z==0
  origin=true
  orpt=roof_group.bounds.min
  pt0=Geom::Point3d.new(0,0,1+(fac*fac))
  tr0=Geom::Transformation.new(pt0)
  roof_group.move!(tr0)
end#if
### --------------------------------------------------------------------
### continue with roof group set up ###---------------------------------
eedges=face.outer_loop.edges
everts=face.outer_loop.vertices###
epoints=[];everts.each{|e|next if not e.valid?; epoints << e.position.to_a}
### remove any inner loop edges in roof group...
edges_in=[];edges_out=[]
roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Edge
    if e.faces[1]### inner loop
      edges_in << e
    else### outer loop
      edges_out << e
    end#if
  end#if
}
edges_in.each{|e|e.erase!} if sof > 0
#edges_out.each{|e|e.erase!} if sof < 0
face=edges_in[0].faces[0] if edges_in[0] && edges_in[0].valid?
face=edges_out[0].faces[0] if edges_out[0] && edges_out[0].valid?
epoints=Roof.ordered_edge_points(face)
eedges=face.outer_loop.edges
eedges.each{|e|e.erase! if e.valid?}
face=roof_entities.add_face(epoints)
eedges=face.outer_loop.edges
everts=face.outer_loop.vertices
### make path and profile etc ###---------------------------------------
dims=[0.mm,nil]###for dist,evert
down=1
everts.each{|evert|
  next if not evert.valid?
  Sketchup::set_status_text("Mansard/Sprocket Roof: Finding Faces "+(100*down/count).to_i.to_s+"%. Please Wait...",SB_PROMPT)
  down=down+1
  dist=1.mm ### = min roof size !
  edge1orig=evert
  edge2orig=evert
  edge1=edge1orig.edges[0]
  edge2=edge2orig.edges[1]
  edge1other=edge1.other_vertex(edge1orig)
  edge2other=edge2.other_vertex(edge2orig)
  vector1=edge1orig.position.vector_to(edge1other.position)
  vector2=edge2orig.position.vector_to(edge2other.position)
  angle1=vector1.to_flat_angle
  angle2=vector2.to_flat_angle
  angle_between=angle1-angle2
  angle=angle_between/2
  pt1=evert.position
  pt2=pt1.offset(vector2,1.mm)
  rotate=Geom::Transformation.rotation(pt1,[0,0,1],angle)
  pt2=pt2.transform!(rotate)
  ### sort which angle gives a point on the face ###
  if angle > 0
    angle=angle+180.degrees if not pt2.is_on_face?(face,true)
  else
    angle=angle-180.degrees if not pt2.is_on_face?(face,true)
  end#if
  ### find biggest one
  on_face=true
  while on_face
    pt2=pt1.offset(vector2,dist)
    rotate=Geom::Transformation.rotation(pt1,[0,0,1],angle)
    pt2=pt2.transform!(rotate)
    if not pt2.is_on_face?(face,true) ### gone off the face
      if dist > dims[0]
        dims=[dist-100.mm,evert]
      end#if
      on_face=false
    end#if
    dist=dist+100.mm###increment dist 100mm till it goes off face.
  end#while
}#end everts.each
### largest dist ###----------------------------------------------------
dist=(dims[0]/1.01).abs#.1570054### never > (sqrt3 - (1/sqrt3)) ### ??????
evert=dims[1]
  edge1orig=evert
  edge2orig=evert
  edge1=edge1orig.edges[0]
  edge2=edge2orig.edges[1]
  edge1other=edge1.other_vertex(edge1orig)
  edge2other=edge2.other_vertex(edge2orig)
  vector1=edge1orig.position.vector_to(edge1other.position)
  vector2=edge2orig.position.vector_to(edge2other.position)
  angle1=vector1.to_flat_angle
  angle2=vector2.to_flat_angle
  angle_between=angle1-angle2
  angle=angle_between/2
  pt1=evert.position
  pt2=pt1.offset(vector2,1.mm)
  rotate=Geom::Transformation.rotation(pt1,[0,0,1],angle)
  pt2=pt2.transform!(rotate)
  ### sort which angle gives a point on the face ###
  if angle > 0
    angle=angle+180.degrees if not pt2.is_on_face?(face,true)
  else
    angle=angle-180.degrees if not pt2.is_on_face?(face,true)
  end#if
  pt2=pt1.offset(vector2,dist)
  rotate=Geom::Transformation.rotation(pt1,[0,0,1],angle)
  pt2=pt2.transform!(rotate)
### sort out transformation ###-----------------------------------------
pt1=Geom::Point3d.new(pt1).transform(roof_group.transformation)if not origin
pt2=Geom::Point3d.new(pt2).transform(roof_group.transformation)if not origin
### need to adjust opp for being on diagonal (sin)
fangle=angle.abs
if fangle >0
 while fangle > 180.degrees
  fangle=fangle-180.degrees
 end#while
end#if
###
### get opposites = height of ridge at dist from pt1
dist1=@ht_/Math::tan(@pitch1_r)
dist2=dist-dist1
opp1=@ht_
opp2=(dist2*Math::sin(fangle)*Math::tan(@pitch2_r)).abs
opp=opp1+opp2
### make upside down triangular cranked face for followme ###-----------
pta=pt1
ptb=[pt1.x,pt1.y,pt1.z+opp]
ptc=[pt2.x,pt2.y,pt2.z+opp]
ptd=[pt1.x+((pt2.x-pt1.x)*(dist1/dist)/Math::sin(fangle)),pt1.y+((pt2.y-pt1.y)*(dist1/dist)/Math::sin(fangle)),pt1.z+opp1]
roof_entities.add_edges(pta,ptb,ptc,ptd)
fface=roof_entities.add_face(pta,ptb,ptc,ptd)
### re-sort so starts from pt1
index=0
0.upto(eedges.length-1) do |i|
  index=i if eedges[i].end.position==pt1
end#upto i
tedges=(eedges.slice(index..-1))+(eedges.slice(0..index-1))
###
fface.followme(tedges)### make main roof faces ###---------------------
### The tidy up ###-----------------------------------------------------
### Intersect :
### The intersect_with method is used to intersect entities, a component
### instance, or group ### with an entities object.
###   entities.intersect_with(recurse,transformation1,entities1,transformation2,hidden,entities2)
### recurse - true if you want this entities object to be recursed .
###   (intersection lines will be put inside of groups and components 
###    within this entities object)
### transformation1 - the transformation for this entities object.
### entities1 - the entities (group etc) where you want the intersection
### lines to appear.
### transformation2 - the transformation for entities1.
### hidden - true if you want hidden geometry in this entities-object to 
### be used in the intersection.
### entities2 - an entities object (or an array of entities ?) = 'faces'.
Sketchup::set_status_text("Mansard/Sprocket: Finding Edges. Please Wait...",SB_PROMPT)
rtr=Geom::Transformation.new()
count.times{
  roof_entities.intersect_with(true,rtr,roof_entities,rtr,true,roof_entities.to_a)
}
### rebuild faces
roof_entities.to_a.each{|e|
  next unless e.valid?
  next unless e.class==Sketchup::Face
  e.reverse! if e.normal.z<0
  nn=e.normal
  eds=e.edges
  e.erase!
  eds.each{|ee|
    next unless ee.valid?
    ee.find_faces
    ee.faces.each{|f|f.reverse! if f.normal==nn.reverse}
  }
}
#UI.messagebox('start')
#return
### It repeats to catch all intersections in complex sets---------------
Sketchup::set_status_text("Mansard/Sprocket Roof: Finishing.",SB_PROMPT)
zz=roof_group.bounds.min.z ### needed later...
rep=(count*count/2).to_i
### remove flat faces...
#rep.times{
 roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Face && e != face
    e.erase! if e.normal.x.abs<1.0e-006 && e.normal.y.abs<1.0e-006
  end#if
 }
#}
Sketchup::set_status_text("Mansard/Sprocket Roof: Finishing..",SB_PROMPT)
### erase all "vertical" edges...
#rep.times{
 roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Edge
    e.erase! if e.start.position.x==e.end.position.x && e.start.position.y==e.end.position.y
  end#if
 }
#}
Sketchup::set_status_text("Mansard/Sprocket Roof: Finishing...",SB_PROMPT)
### erase faceless edges...
#rep.times{
 roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Edge
    e.erase! if ! e.faces[0]
  end#if
 }
#}
Sketchup::set_status_text("Mansard/Sprocket Roof: Finishing....",SB_PROMPT)
### remove one face edges = off base...
#rep.times{
 roof_entities.to_a.each{|e|
  #Sketchup.active_model.active_view.refresh
  if e.valid? and e.class==Sketchup::Edge and e.bounds.min.z!=zz
    e.erase! if ! e.faces[1]
  end#if
 }
#}
Sketchup::set_status_text("Mansard/Sprocket Roof: Finishing.....",SB_PROMPT)
### now reverse faces so 'up'
roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Face
    e.reverse! if e.normal.z <0
  end#if
}
Sketchup::set_status_text("Mansard/Sprocket Roof: Finishing......",SB_PROMPT)
### remove all 'flat' faces again...
#rep.times{
 roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Face
    e.erase! if e.normal.x.abs<1.0e-006 && e.normal.y.abs<1.0e-006
  end#if
 }
#}
Sketchup::set_status_text("Mansard/Sprocket Roof: Finishing.......",SB_PROMPT)
### trap any non-base non-flat faces...
faces_hi=[]
roof_entities.to_a.each{|e|
  attached=true
  if e.valid? and e.class==Sketchup::Face && e.bounds.min.z > zz
    e.edges.each{|ee|
      zzz=e.bounds.min.z ### bottom edges
      if ee.valid? and ee.bounds.min.z==zzz && ee.start.position.z==ee.end.position.z ### it is level
        otherfaces=ee.faces-[e]
        otherfaces.each{|f|
          attached=false if f.bounds.min.z != zz
        }
      end#if
    }
    faces_hi << e if not attached
  end#if
}
faces_hi.each{|f|
  coplanar=false
  f.edges.each{|e|
    coplanar=true if e.faces[1] && e.faces[0].normal==e.faces[1].normal
    coplanar=true if e.faces[2] && e.faces[0].normal==e.faces[2].normal
    coplanar=true if e.faces[2] && e.faces[1].normal==e.faces[2].normal
    coplanar=true if e.faces[3] && e.faces[0].normal==e.faces[3].normal
    coplanar=true if e.faces[3] && e.faces[1].normal==e.faces[3].normal
    coplanar=true if e.faces[3] && e.faces[2].normal==e.faces[3].normal
  }
  f.erase! if f.valid? && ! coplanar
}
faces_hi.each{|f|
  if f.valid?
    f.edges.each{|e|
      e.erase! if e.valid? && ! e.faces[1]
    }
  end#if
}
Sketchup::set_status_text("Mansard/Sprocket Roof: Finishing.......",SB_PROMPT)
### tidy up faceless and coplanar edges etc...
#rep.times{
 roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Edge
    e.erase! if ! e.faces[0]### faceless
  end#if 
  if e.valid? && e.class==Sketchup::Edge
    e.erase! if e.faces[1] && e.faces[0].normal==e.faces[1].normal###coplanar
  end#if
 }
#}
Sketchup::set_status_text("Mansard/Sprocket Roof: Finishing........",SB_PROMPT)
### remove one face edges that are above base and only have one face...
#rep.times{
 roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Face
    if e.bounds.min.z > zz
      edges=e.edges
      edges.each{|ee|ee.erase! if ee.valid? && ! ee.faces[1]}
    end#if
  end#if
 }
#}
Sketchup::set_status_text("Mansard/Sprocket Roof: Finishing.........",SB_PROMPT)
### erase any 'floating' bits of roof above the main body...
#rep.times{
 roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Face && faces_hi.include?(e)
    if e.bounds.min.z > zz
      edges=e.edges
      edges.each{|ee|ee.erase! if ee.valid?}
    end#if
  end#if
 }
#}
Sketchup::set_status_text("Mansard/Sprocket Roof: Finished.",SB_PROMPT)
### erase faceless edges again...
#rep.times{
 roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Edge
    e.erase! if ! e.faces[0]
  end#if
 }
#}
### trap for failure
broken=false
roof_entities.each{|e|
  if e.class==Sketchup::Edge and e.line[1].z.abs>=0.001 and not e.faces[1]
    broken=true
    break
  end#if
}
roof_entities.erase_entities(roof_entities.to_a) if broken
### if fails...
if ! roof_entities[0] || roof_entities.length <= count*2
  UI.messagebox("Mansard/Sprocket Roof Error:\n\nIt was not possible to make a single pitch mansard/sprocket roof from this face.\n\nSometimes the face/edge relationships get muddled in the model's database.\nTry re-making the original face:\n- Delete it and redraw over one edge to remake it.  \nOr copy the face to one side, delete the original and move the copy back.  \nThis methods can often make a face 'roof'.\n\nIf not, then try sub-dividing the face into pieces and joining their roofs manually.\n\nNote that some face shapes can never be roofed with a single pitch.\ne.g. crooked stars.\n\nExiting...")
  roof_group.erase! if roof_group.valid?
  ### tidy up at end ###--------------------------------------------------
  @model.selection.clear
  @model.selection.add(oface)### end with original face re-selected
  @model.active_view.invalidate
  @model.commit_operation
  Sketchup.send_action"selectSelectionTool:"
  return nil
end#if
###---------------------------------------------------------------------
### correct for fascia
roof_group.move!(Geom::Transformation.new([0,0,fac_up]))if fac_up>0
###
### sort out soffit ###-------------------------------------------------
soffit_group=entities.add_group###v3.0
soffit_entities=soffit_group.entities###v3.0
face=soffit_entities.add_face(opoints)###v3.0
if sof!=0 ###v3.0
  sface=face.offset_eaves(sof)
  verts=face.vertices
  sverts=sface.outer_loop.vertices
  sface.reverse! if sface.normal.z>0
  face.erase! if face.valid?### make the hole
end#if
###
if sof!=0 && sof_up>0
  ### fold soffit if sloping...
 verts.each{|v|
  next if not v.valid?
  pos=v.position
  posn=[pos.x,pos.y,pos.z+sof_up]
  vec=pos.vector_to(posn)
  tran=Geom::Transformation.translation(vec)
  soffit_entities.transform_entities(tran,v)
 }
 ### erase coplanar edges.
 soffit_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Edge
    if e.faces[1] and not e.faces[2]
	  e.erase! if e.faces[0].normal==e.faces[1].normal and e.valid?
	end#if
	e.erase! if e.valid? and not e.faces[0]
  end#if
 }###
end#if sof_up
### move whole roof group down by sof_up so it sits on selected face.
trz=Geom::Transformation.new([0,0,sof_up*-1+fac_up])if sof_up>0###v3.0
roof_group.move!(trz)if sof_up>0###v3.0
trz=Geom::Transformation.new([0,0,sof_up*-1])if sof_up>0###v3.0
soffit_group.move!(trz)if sof_up>0###v3.0
###v3.0
### sort out fascia etc ###----------------------------------------------
fascia_group=entities.add_group
fascia_entities=fascia_group.entities
face=fascia_entities.add_face(opoints)
###
if fac_out==0
  fface=face.offset_eaves(sof) if sof!=0
  fface=face if sof==0
  xverts=fface.outer_loop.vertices
  xxpoints=[]
  xverts.each{|e|
    next if not e.valid?
    ee=e.position.to_a
    xxpoints << [ee[0],ee[1],ee[2]+fac_up]
  }
  edges2go=face.edges if sof!=0
  edges2go.each{|e|e.erase!} if sof!=0
  fascia_entities.add_face(xxpoints)
  fascia_entities.to_a.each{|e|e.erase! if e.valid? and e.class==Sketchup::Face}
  vlines=[]
  xverts.each{|e|
    next if not e.valid?
    ee=e.position.to_a
    v=fascia_entities.add_line(ee,[ee[0],ee[1],ee[2]+fac_up])
    vlines << v
  }
  vlines.each{|e|e.find_faces if e}
  fascia_entities.to_a.each{|e|e.reverse! if e.valid? and e.class==Sketchup::Face and oface.bounds.min.z!=0}
else ### fac_out > 0 ==sloping
  fface=face.offset_eaves(sof) if sof!=0
  fface=face if sof==0
  edges2go=face.edges if sof!=0
  ffface=fface.offset_eaves(fac_out)
  ffface.reverse! if ffface.normal.z>0
  fface.erase! if fface.valid?
  face.erase! if face.valid?
  edges2go.each{|e|e.erase! if e.valid?} if sof!=0
  fverts=ffface.outer_loop.vertices
  fverts.each{|v|
    next if not v.valid?
    pos=v.position
    posn=[pos.x,pos.y,pos.z+fac_up]
    vec=pos.vector_to(posn)
    tran=Geom::Transformation.translation(vec)
    fascia_entities.transform_entities(tran,v)
  }
  ### erase coplanar edges.
  fascia_entities.to_a.each{|e|
    if e.valid? and e.class==Sketchup::Edge
      if e.faces[1] and not e.faces[2]
	    e.erase! if e.faces[0].normal==e.faces[1].normal and e.valid?
	  end#if
	  e.erase! if e.valid? and not e.faces[0]
    end#if
  }
end#if
### move whole fascia group down by sof_up so it sits on selected face.
trz=Geom::Transformation.new([0,0,sof_up*-1])
fascia_group.move!(trz)
### move origin based faces etc back into place to avoid bugsplats ###--
if origin
  tr=Geom::Transformation.new(orpt)
  trz=Geom::Transformation.new([0,0,0])
  roof_group.move!(trz)
  ### correct for fascia ?
  roof_group.move!(Geom::Transformation.new([0,0,fac_up]))if fac_up>0
  soffit_group.move!(trz)
  fascia_group.move!(trz)
end#if
###v3.0
### Moved materials dialog to after roof is made...
def Roof::dialog_mats_man ()
### materials dialog ###------------------------------------------------
  model=Sketchup.active_model
  ### layers...
  mlayers=model.layers
  alayer=model.active_layer
  model.active_layer=nil
  deflayer=model.active_layer.name
  model.active_layer=alayer
  rlayer="-ROOF"### edit this to suit your desired default roof layer.
  layers=[]
  mlayers.each{|layer|layers << layer.name if layer.name!=deflayer}
  layers=[rlayer,deflayer]+layers.sort!
  layers=layers.uniq.join('|')
  ### materials...
  mat_roof="Roof Surface"
  mat_fascia="Roof Fascia"
  mat_soffit="Roof Soffit"
  materials=model.materials
  mats=[]
  materials.each{|mat|mats << mat.display_name}
  mats.sort!
  list_mat_roof=((["<Default>"]+[mat_roof]+mats).uniq).join("|")
  list_mat_fascia=((["<Default>"]+[mat_fascia]+mats).uniq).join("|")
  list_mat_soffit=((["<Default>"]+[mat_soffit]+mats).uniq).join("|")
  ### set up variables...
  @layer=rlayer if ! @layer
  @mat_roof=mat_roof if ! @mat_roof
  @mat_fascia=mat_fascia if ! @mat_fascia
  @mat_soffit=mat_soffit if ! @mat_soffit
  enums=[layers,list_mat_roof,list_mat_fascia,list_mat_soffit]
  prompts=["Roof Layer:  ","Roof Surface:  ","Roof Fascia:  ","Roof Soffit: "]
  values=[@layer,@mat_roof,@mat_fascia,@mat_soffit]
  @other_results=(inputbox prompts,values,enums,"Roof Layer and Materials")
  @layer,@mat_roof,@mat_fascia,@mat_soffit=@other_results if @other_results
  @other="No" if ! @other_results
  ###
end#def
### run material dialog
  Roof::dialog_mats_man() if @other=="Yes"
### 
###v3.0
### sort out layer, materials etc ###-----------------------------------
if @other=="Yes"
  ### note that layer is set after groups combined...
  extg=nil;@model.layers.each{|lyr|extg=lyr if lyr.name==@layer}
  if @layer=="<Default>"
    layr=nil
  else
    if extg
      layr=extg
    else
      layr=@model.layers.add(@layer)
    end#if
  end#if
  ### materials...
  ### roof
  extg=nil;@model.materials.each{|mat|extg=mat if mat.display_name==@mat_roof}
  if @mat_roof=="<Default>"
    mat_r=nil
  else
    if extg
      mat_r=extg
    else
      mat_r=@model.materials.add(@mat_roof)
      mat_r.color=[111,111,111]### adjust default colours here.
    end#if
  end#if
  roof_entities.to_a.each{|e|e.material=mat_r if e.valid? and e.class==Sketchup::Face}
  ### fascia
  extg=nil;@model.materials.each{|mat|extg=mat if mat.display_name==@mat_fascia}
  if @mat_fascia=="<Default>"
    mat_f=nil
  else
    if extg
      mat_f=extg
    else
      mat_f=@model.materials.add(@mat_fascia)
      mat_f.color=[66,66,66]### adjust default colours here.
    end#if
  end#if
  fascia_entities.to_a.each{|e|e.material=mat_f if e.valid? and e.class==Sketchup::Face}
  ### soffit
  extg=nil;@model.materials.each{|mat|extg=mat if mat.display_name==@mat_soffit}
  if @mat_soffit=="<Default>"
    mat_s=nil
  else
    if extg
      mat_s=extg
    else
      mat_s=@model.materials.add(@mat_soffit)
      mat_s.color=[222,222,222]### adjust default colours here.
    end#if
  end#if
  soffit_entities.to_a.each{|e|e.material=mat_s if e.valid? and e.class==Sketchup::Face}
  ###
end#if
### combine pieces ###--------------------------------------------------
tgroup=entities.add_group(roof_group,soffit_group,fascia_group)
roof_group.explode
soffit_group.explode
fascia_group.explode
roof_group=tgroup
###
roof_group.layer=layr if @other=="Yes"### layer set here...
### tidy up at end ###--------------------------------------------------
@model.selection.clear
@model.selection.add(oface)### end with original face re-selected
@model.active_view.invalidate
@model.commit_operation
Sketchup.send_action"selectSelectionTool:"
  ###
end#def mansard











def Roof::gable(pts)
	@pts=pts
	Sketchup::set_status_text "Roof Gable:"
    @model=Sketchup.active_model
	entities=@model.active_entities
	p1=pts[0]; p2=pts[1]; pd=pts[2]
	z1=p1.z; p2.z=z1; pd.z=z1
	if (p1==p2 || p2==pd) || (pd==p1)
	  UI.messagebox("Roof Gable:\nCan't have any points in the same location in plan !\nTry again...")
	  Sketchup.send_action "selectSelectionTool:"
	  return nil
	end#if
	ang12=(p1.vector_to(p2)).to_flat_angle
	ang2d=(p2.vector_to(pd)).to_flat_angle
	ang=(ang2d-ang12).to_f
	if ang.abs.is_near_to?(0.0.degrees) || ang.abs.is_near_to?(180.0.degrees)
	  UI.messagebox("Roof Gable:\nCan't have 3 points in a line !\nTry again...")
	  Sketchup.send_action "selectSelectionTool:"
	  return nil
	end#if
### do gable dialog...
def Roof::dialog_gable ()
### dialog ###----------------------------------------------------------
@pitchG="1:2" if ! @pitchG && @model.options["UnitsOptions"]["LengthUnit"]<2
@pitchG="30" if ! @pitchG && @model.options["UnitsOptions"]["LengthUnit"]>1
@fascia=8.inch if ! @fascia && @model.options["UnitsOptions"]["LengthUnit"]<2
@fascia=200.mm if ! @fascia && @model.options["UnitsOptions"]["LengthUnit"]>1
@soffit=16.inch if ! @soffit && @model.options["UnitsOptions"]["LengthUnit"]<2
@gable1=400.mm if ! @gable1 && @model.options["UnitsOptions"]["LengthUnit"]>1
@gable1=16.inch if ! @gable1 && @model.options["UnitsOptions"]["LengthUnit"]<2
@gable2=400.mm if ! @gable2 && @model.options["UnitsOptions"]["LengthUnit"]>1
@gable2=16.inch if ! @gable2 && @model.options["UnitsOptions"]["LengthUnit"]<2
@soffit=400.mm if ! @soffit && @model.options["UnitsOptions"]["LengthUnit"]>1
@gable1type="Verge from Fascia" if ! @gable1type
@gable2type="Verge from Fascia" if ! @gable2type
@fstype="Vertical+Horizontal" if ! @fstype###v3.0
fstypes="Vertical+Horizontal|Vertical+Sloping|Sloping+Sloping|Sloping+Horizontal"###v3.0
@other="No" if ! @other
@units="inches" if @model.options["UnitsOptions"]["LengthUnit"]==0
@units="feet" if @model.options["UnitsOptions"]["LengthUnit"]==1
@units="mm" if @model.options["UnitsOptions"]["LengthUnit"]==2
@units="cm" if @model.options["UnitsOptions"]["LengthUnit"]==3
@units="m" if @model.options["UnitsOptions"]["LengthUnit"]==4
gabletypes="Blank|Blank with Fascia|Verge from Fascia|Verge from Soffit|Verge by Height|Verge by Width|Verge by Height, with Fascia|Verge by Width, with Fascia|Verge with Fascia [All Equal]|Verge by Height, with Box-Eaves|Verge by Width, with Box-Eaves"
enums=["","","",fstypes,"","",gabletypes,gabletypes,"Yes|No"]###v3.0
prompts=["Roof Slope (in Degrees or A:B or ^Height(#{@units})):  ","Eaves Fascia Size(#{@units}):  ","Eaves Soffit Size(#{@units}):  ","Fascia+Soffit Type:  ","First Gable Size(#{@units}): ","Second Gable Size(#{@units}): ","First Gable Type: ","Second Gable Type: ","Set Layer and Materials ? :"]###v3.0
values=[@pitchG,@fascia,@soffit,@fstype,@gable1,@gable2,@gable1type,@gable2type,@other]###v3.0
@results=(inputbox prompts,values,enums,"Gable Roof Settings")
(Sketchup.send_action "selectSelectionTool:"; return nil) if ! @results
(@pitch_s,@fascia_,@soffit_,@fstype_,@gable1_,@gable2_,@gable1type_,@gable2type_,@other_)=@results
@pitchG=@pitch_s;@fascia=@fascia_;@soffit=@soffit_;@fstype=@fstype_;###v3.0
@gable1=@gable1_;@gable2=@gable2_;@gable1type=@gable1type_;@gable2type=@gable2type_;@other=@other_
### sort if ^height or slope...
if @pitch_s.split("")[0]=="^"
  ght=@pitch_s[1..-1].to_f
  ght=ght.to_l if @units=="inches"
  ght=ght.feet if @units=="feet"
  ght=ght.mm if @units=="mm"
  ght=ght.cm if @units=="cm"
  ght=ght.m if @units=="m"
  adj=(@pts[0].distance(@pts[1]))/2 + @soffit #adj=(@oedge.length/2)+@soffit
  @pitch_d=(Math::atan(ght/adj)).radians
else#it's an angle
  ### get angle of roof in radians
  ratio=@pitch_s.split(":").to_f
  if ratio[1]
    tan=ratio[0]/ratio[1]
    @pitch_d=(Math::atan(tan)).radians
  else
    @pitch_d=ratio[0]
  end#if
end#if
@pitch_r=@pitch_d.degrees  ### to radians ###------------------------
if @pitch_d<1
  UI.messagebox("Gable Roof:\n\nPitch can't be less than 1 degree.")
  @pitchG="1";@pitch_s="1";@pitch_d=1;@pitch_r=1.degrees
  Roof::dialog_gable
  return nil
end#if
if @pitch_d>89 
  UI.messagebox("Gable Roof:\n\nPitch can't be more than 89 degrees.")
  @pitchG="89";@pitch_s="89";@pitch_d=89;@pitch_r=89.degrees
  Roof::dialog_gable
  return nil
end#if
if @fascia_ <0
  UI.messagebox("Gable Roof:\n\nFascia can't be LESS THAN ZERO.\nResetting it to ZERO...")
  @fascia=0.to_l;@fascia_=0.to_l
  Roof::dialog_gable
  return nil
end#if
if @soffit_ <0
  UI.messagebox("Gable Roof:\n\nEaves can't be LESS THAN ZERO.\nResetting it to ZERO...\n\nFor a 'parapet-eaves' first use 'Offset' on the face to inset its edges ...")
  @soffit=0.to_l;@soffit_=0.to_l
  Roof::dialog_gable
  return nil
end#if
###
if @gable1_ <0
  UI.messagebox("Gable Roof:\n\nFirst Gable can't be LESS THAN ZERO.\nResetting it to ZERO...\n\nFor a 'parapet-gable' first use 'Offset' on the face to inset its edges ...")
  @soffit=0.to_l;@soffit_=0.to_l
  Roof::dialog_gable
  return nil
end#if
if @gable2_ <0
  UI.messagebox("Gable Roof:\n\Second Gable can't be LESS THAN ZERO.\nResetting it to ZERO...\n\nFor a 'parapet-gable' first use 'Offset' on the face to inset its edges ...")
  @soffit=0.to_l;@soffit_=0.to_l
  Roof::dialog_gable
  return nil
end#if
if @fascia_==0 && (@gable1type_=="Verge from Fascia"||@gable1type_=="Verge with Fascia [All Equal]")
  UI.messagebox("Gable Roof:\n\nFascia is ZERO.\nTherefore cannot make 'Verge from Fascia' ...")
  @gable1type_="Blank";@gable1type="Blank"
  Roof::dialog_gable
  return nil
end#if
if @fascia_==0 && (@gable2type_=="Verge from Fascia"||@gable2type_=="Verge with Fascia [All Equal]")
  UI.messagebox("Gable Roof:\n\nFascia is ZERO.\nTherefore cannot make 'Verge from Fascia' ...")
  @gable2type_="Blank";@gable2type="Blank"
  Roof::dialog_gable
  return nil
end#if
if @soffit_==0 && @gable1type_=="Verge from Soffit"
  UI.messagebox("Gable Roof:\n\Soffit is ZERO.\nTherefore cannot make 'Verge from Soffit' ...")
  @gable1type_="Blank";@gable1type="Blank"
  Roof::dialog_gable
  return nil
end#if
if @soffit_==0 && @gable2type_=="Verge from Soffit"
  UI.messagebox("Gable Roof:\n\Soffit is ZERO.\nTherefore cannot make 'Verge from Soffit' ...")
  @gable2type_="Blank";@gable2type="Blank"
  Roof::dialog_gable
  return nil
end#if
if @fascia_==0 && @gable1type=="Verge by Height, with Fascia"
  UI.messagebox("Gable Roof:\n\nFascia is ZERO.\nTherefore cannot make 'Verge by Height, with Fascia' ...")
  @gable1type_="Blank";@gable1type="Blank"
  Roof::dialog_gable
  return nil
end#if
if @fascia_==0 && @gable2type=="Verge by Height, with Fascia"
  UI.messagebox("Gable Roof:\n\nFascia is ZERO.\nTherefore cannot make 'Verge by Height, with Fascia' ...")
  @gable2type_="Blank";@gable2type="Blank"
  Roof::dialog_gable
  return nil
end#if
if @fascia_==0 && @gable1type=="Blank with Fascia"
  UI.messagebox("Gable Roof:\n\nFascia is ZERO.\nTherefore cannot make 'Blank with Fascia' ...")
  @gable1type_="Blank";@gable1type="Blank"
  Roof::dialog_gable
  return nil
end#if
if @fascia_==0 && @gable2type=="Blank with Fascia"
  UI.messagebox("Gable Roof:\n\nFascia is ZERO.\nTherefore cannot make 'Blank with Fascia' ...")
  @gable2type_="Blank";@gable2type="Blank"
  Roof::dialog_gable
  return nil
end#if
if (@fascia_==0 && @soffit==0) && (@gable1type=="Verge by Height, with Box-Eaves"||@gable1type=="Verge by Width, with Box-Eaves")
  UI.messagebox("Gable Roof:\n\nFascia AND Soffit are ZERO.\nTherefore cannot make a 'Box Eaves' ...")
  @gable1type_="Blank";@gable1type="Blank"
  Roof::dialog_gable
  return nil
end#if
if (@fascia_==0 && @soffit==0) && (@gable2type=="Verge by Height, with Box-Eaves"||@gable2type=="Verge by Width, with Box-Eaves")
  UI.messagebox("Gable Roof:\n\nFascia AND Soffit are ZERO.\nTherefore cannot make a 'Box Eaves' ...")
  @gable2type_="Blank";@gable2type="Blank"
  Roof::dialog_gable
  return nil
end#if
###v3.0
if @fascia_ ==0 && @fstype_=="Vertical+Sloping"
  UI.messagebox("Gable Roof:\n\nIf the Fascia is ZERO then you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
  @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
  Roof::dialog_gable
  return nil
end#if
if @fascia_ ==0 && @fstype_=="Sloping+Sloping"
  UI.messagebox("Gable Roof:\n\nIf the Fascia is ZERO then you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
  @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
  Roof::dialog_gable
  return nil
end#if
###----------------------------------
### CRIB: gabletypes="Blank|Blank with Fascia|Verge from Fascia|Verge from Soffit|Verge by Height|Verge by Width|Verge by Height, with Fascia|Verge by Width, with Fascia|Verge with Fascia [All Equal]|Verge by Height, with Box-Eaves|Verge by Width, with Box-Eaves"###v3.0 tests...
if @fstype=~/Sloping\+/
  if @gable1type=="Blank"
    if @gable2type=="Blank"
	  ###OK
	end#if
    if @gable2type=="Blank with Fascia" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'Blank with Fascia' with a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Blank with Fascia" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'Blank with Fascia' with a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge from Fascia"
	  ###OK
	end#if
    if @gable2type=="Verge from Soffit" && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'from Soffit' you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge from Soffit" && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'from Soffit' you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Fascia" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Fascia" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Fascia" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Fascia" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge with Fascia [All Equal]" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'with Fascia [All Equal]' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge with Fascia [All Equal]" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'with Fascia [All Equal]' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Box-Eaves" && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Box-Eaves" && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Box-Eaves" && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Box-Eaves" && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
  end#if Blank
### ### ###----------------------------
  if @gable1type=="Blank with Fascia"
    if @gable1==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 1std Verge is 'Blank with Fascia' with a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable1==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 1std Verge is 'Blank with Fascia' with a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Blank"
	  ###OK
	end#if
    if @gable2type=="Blank with Fascia" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'Blank with Fascia' with a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Blank with Fascia" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'Blank with Fascia' with a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge from Fascia"
	  ###OK
	end#if
    if @gable2type=="Verge from Soffit" && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'from Soffit' you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge from Soffit" && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'from Soffit' you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Fascia" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Fascia" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Fascia" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Fascia" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge with Fascia [All Equal]" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'with Fascia [All Equal]' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge with Fascia [All Equal]" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'with Fascia [All Equal]' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Box-Eaves" && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Box-Eaves" && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Box-Eaves" && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Box-Eaves" && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
  end#if Blank with Fascia
### ### ###----------------------------
  if @gable1type=="Verge from Fascia"
    if @gable2type=="Blank"
	  ###OK
	end#if
    if @gable2type=="Blank with Fascia" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'Blank with Fascia' with a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Blank with Fascia" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'Blank with Fascia' with a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge from Fascia"
	  ###OK
	end#if
    if @gable2type=="Verge from Soffit" && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'from Soffit' you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge from Soffit" && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'from Soffit' you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Fascia" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Fascia" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Fascia" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Fascia" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge with Fascia [All Equal]" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'with Fascia [All Equal]' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge with Fascia [All Equal]" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'with Fascia [All Equal]' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Box-Eaves" && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Box-Eaves" && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Box-Eaves" && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Box-Eaves" && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
  end#if Verge from Fascia
### ### ###----------------------------
  if @gable1type=="Verge from Soffit"
    if @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 1st Verge is 'from Soffit' you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 1st Verge is 'from Soffit' you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
  end#if Verge from Soffit
### ### ###----------------------------
  if @gable1type=="Verge by Height" || @gable1type=="Verge by Width"
    if @gable2type=="Blank"
	  ###OK
	end#if
    if @gable2type=="Blank with Fascia" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'Blank with Fascia' with a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Blank with Fascia" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'Blank with Fascia' with a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge from Fascia" 
	  ###OK
	end#if
    if @gable2type=="Verge from Soffit" && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'from Soffit' you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge from Soffit" && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'from Soffit' you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Fascia" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Fascia" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Fascia" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Fascia" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge with Fascia [All Equal]" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'with Fascia [All Equal]' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge with Fascia [All Equal]" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'with Fascia [All Equal]' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Box-Eaves" && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Box-Eaves" && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Box-Eaves" && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Box-Eaves" && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
  end#if Verge by Height / Width
### ### ###----------------------------
  if (@gable1type=="Verge by Height, with Fascia" || @gable1type=="Verge by Width, with Fascia") || @gable1type=="Verge with Fascia [All Equal]"
    if @gable1==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 1st Verge is '...with Fascia...', BUT with a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable1==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 1st Verge is '...with Fascia...', BUT with a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Blank"
	  ###OK
	end#if
    if @gable2type=="Blank with Fascia" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'Blank with Fascia' with a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Blank with Fascia" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'Blank with Fascia' with a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge from Fascia" 
	  ###OK
	end#if
    if @gable2type=="Verge from Soffit" && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'from Soffit' you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge from Soffit" && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'from Soffit' you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Fascia" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Fascia" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Fascia" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Fascia" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Fascia' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge with Fascia [All Equal]" && @gable2==0 && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'with Fascia [All Equal]' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge with Fascia [All Equal]" && @gable2==0 && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'with Fascia [All Equal]' and has a ZERO overhang, \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Box-Eaves" && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Height, with Box-Eaves" && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Height, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Box-Eaves" && @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @gable2type=="Verge by Width, with Box-Eaves" && @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 2nd Verge is 'by Width, with Box-Eaves', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
  end#if Verge ...with Fascia...
  ### ### ###----------------------------
  if @gable1type=~/with Box-Eaves/
    if @fstype=="Vertical+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 1st Verge is '...with Box-Eaves...', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
      @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
    if @fstype=="Sloping+Sloping"
	  UI.messagebox("Gable Roof:\n\nIf the 1st Verge is '...with Box-Eaves...', \nthen you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
      @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
      Roof::dialog_gable
      return nil
	end#if
  end#if ...with Box-Eaves...
  ###
end#if sloping sof -----------------------------
### CRIB: gabletypes="Blank|Blank with Fascia|Verge from Fascia|Verge from Soffit|Verge by Height|Verge by Width|Verge by Height, with Fascia|Verge by Width, with Fascia|Verge with Fascia [All Equal]|Verge by Height, with Box-Eaves|Verge by Width, with Box-Eaves"###v3.0 tests...
###v3.0 ###-----------------------------------
##############################################
end#def dialog
### run dialog
  Roof::dialog_gable()
  (Sketchup.send_action "selectSelectionTool:"; return nil) if ! @results
###
###v3.0 sort out sloping fascia and soffit
Sketchup::set_status_text "Roof Gable: Processing..."
### defaults=Vertical+Horizontal
fac=@fascia; fac_up=fac; fac_out=0.0
sof=@soffit; sof_up=0.0; sf_up=0.0### adjusts ridge ht for sloping fascia
if @fstype=~/^Sloping+/
  fac_up = fac*Math::cos(@pitch_r)
  fac_out= fac*Math::sin(@pitch_r)
  sf_up= fac_out*Math::tan(@pitch_r)
end#if
if @fstype=~/Sloping$/
  sof_up= sof*Math::tan(@pitch_r)
end#if
###v3.0
###---------------------------------------------------------------------
	cwise=false
	cwise=true if (ang12>=0.0.degrees && ang12<90.0.degrees) && (ang2d<ang12 || ang2d>ang12+180.0.degrees)
	cwise=true if (ang12>=90.0.degrees && ang12<180.0.degrees) && (ang2d>ang12 && ang2d<ang12+180.0.degrees)
	cwise=true if (ang12>=180.0.degrees && ang12<270.0.degrees) && (ang2d<ang12 || ang2d<ang12-180.0.degrees)
	cwise=true if (ang12>=270.0.degrees && ang12<=360.0.degrees) && (ang2d<ang12 && ang2d>ang12-180.0.degrees)
	ang90=90.0.degrees
	#ang90=-ang90 if cwise
	(pt=p1;p1=p2;p2=pt) if cwise
	###
	roof_group=entities.add_group
	roof_entities=roof_group.entities
	ln1=roof_entities.add_line(p1,p2)
	###len=ln1.length ### width of base
    ### get dist of p3d from line of ln1,this is depth of base
	dist=pd.distance_to_line(ln1.line)
	ln1.erase! if ln1.valid?
	### add p3 along vect p2,p1 at dist,rotate p3 90deg
	vec=p2.vector_to(p1)
	p3=p2.offset(vec,dist)
	rotate=Geom::Transformation.rotation(p2,[0,0,1],-ang90)
	p3=p3.transform!(rotate)
	#ln2=roof_entities.add_line(p2,p3)
	###add p4 along vect p1,p2 at dist,rotate p4 -90deg
	vec=p1.vector_to(p2)
	p4=p1.offset(vec,dist)
	rotate=Geom::Transformation.rotation(p1,[0,0,1],ang90)
	p4=p4.transform!(rotate)
	### p1,p2,p3,p4 are corners of base 'hole'
    ### DO GABLE...
	### now we find corners of 1st gable
	vec=p4.vector_to(p1)
	g1=p1.offset(vec,@gable1)
	vec=p3.vector_to(p2)
	g2=p2.offset(vec,@gable1)
	vec=g2.vector_to(g1)
	g1=g1.offset(vec,sof)
	vec=g1.vector_to(g2)
	g2=g2.offset(vec,sof)
	### get centre
cp=Geom::Point3d.linear_combination(0.5,g1,0.5,g2)
adj=g1.distance(cp)##########################
opp=adj*Math::tan(@pitch_r)##################
### start undo
@model.start_operation("Gable Roof")
### make profile
vec=p1.vector_to(p2)
p12f=g1.offset(vec, g1.distance(g2)+fac_out)
p12f=[p12f.x,p12f.y,p12f.z+fac_up]
vec=p2.vector_to(p1)
p21f=g2.offset(vec, g2.distance(g1)+fac_out)
p21f=[p21f.x,p21f.y,p21f.z+fac_up]
### add lines
g12=roof_entities.add_line(g1,g2)
fe1=roof_entities.add_line(g2,p12f)if fac_up!=0.0
ge1=roof_entities.add_line(p12f,[cp.x,cp.y,cp.z+opp+fac_up+sf_up])
ge2=roof_entities.add_line([cp.x,cp.y,cp.z+opp+fac_up+sf_up],p21f)
fe2=roof_entities.add_line(p21f,g1)if fac!=0.0
### draw gable1
if fac==0.0
  gface=roof_entities.add_face(g12,ge1,ge2)
else
  gface=roof_entities.add_face(g12,fe1,ge1,ge2,fe2)
end#if#
gable1=gface### this is 1st gable
gface.pushpull((-@gable1-dist-@gable2),false)### make roof shape
###-----------------------------------------
### get parts of roof
roof_faces=[];gables=[]
### make hole in base
roof_entities.to_a.each{|e|roof_faces << e if e.valid? and e.class==Sketchup::Face}
roof_faces.each{|face|
  gables << face if face.normal.z.is_near_to?(0.0) && face.edges.length!=4
}
gable2=(gables-[gable1])[0]
gable1normal=gable1.normal
gable2normal=gable2.normal
###
gedge1=nil
gable1.edges.each{|e|
  gedge1=e if e.start.position.z==e.end.position.z
}
gs1=gedge1.start.position; ge1=gedge1.end.position
cp1=Geom::Point3d.linear_combination(0.5,gs1,0.5,ge1)
gs1=gs1.offset(ge1.vector_to(gs1),fac_out)###v3.0
ge1=ge1.offset(gs1.vector_to(ge1),fac_out)###v3.0
gedge2=nil
gable2.edges.each{|e|
  gedge2=e if e.start.position.z==e.end.position.z
}
gs2=gedge2.start.position; ge2=gedge2.end.position
cp2=Geom::Point3d.linear_combination(0.5,gs2,0.5,ge2)
gs2=gs2.offset(ge2.vector_to(gs2),fac_out)###v3.0
ge2=ge2.offset(gs2.vector_to(ge2),fac_out)###v3.0
if @gable1type=="Blank with Fascia"
  ln1=roof_entities.add_line([gs1.x,gs1.y,gs1.z+fac_up],[ge1.x,ge1.y,ge1.z+fac_up])####
end#if
if @gable2type=="Blank with Fascia"
  ln1=roof_entities.add_line([gs2.x,gs2.y,gs2.z+fac_up],[ge2.x,ge2.y,ge2.z+fac_up])####
end#if
### make gable types
### = Blank|Blank with Fascia|Eaves into Verge|Soffit into Verge etc
### gable ONE first...
if @gable1type=="Verge from Fascia"
  if @fstype=~/^Sloping+/
    gs1=gs1.offset(gs1.vector_to(ge1),fac_out)
    ge1=ge1.offset(ge1.vector_to(gs1),fac_out)
  end#if
  ln1=roof_entities.add_line(ge1,[cp1.x,cp1.y,cp1.z+opp])
  roof_entities.add_line([cp1.x,cp1.y,cp1.z+opp],gs1)
  gab1=nil; ln1.faces.each{|e|
    gab1=e if e.bounds.max.z!=roof_group.bounds.max.z
  }
  gab1.pushpull(-@gable1,false) if gab1
elsif @gable1type=="Verge from Soffit"
  if @fstype=~/^Sloping+/
    gs1=gs1.offset(gs1.vector_to(ge1),fac_out)
    ge1=ge1.offset(ge1.vector_to(gs1),fac_out)
  end#if
  rise=Math::tan(@pitch_r)*sof###
  ge1=ge1.offset(ge1.vector_to(gs1),sof)
  gs1=gs1.offset(gs1.vector_to(ge1),sof)
  ln1=roof_entities.add_line(ge1,[cp1.x,cp1.y,cp1.z+opp-rise])###
  roof_entities.add_line([cp1.x,cp1.y,cp1.z+opp-rise],gs1)###
  gab1=nil; ln1.faces.each{|e|
    gab1=e if e.bounds.max.z!=roof_group.bounds.max.z
  }
  gab1.pushpull(-@gable1,false) if gab1
  ###Verge by Height|Verge by Width|Verge by Height, with Fascia|Verge by Width, with Fascia|Verge by Height, with Box-Eaves|Verge by Width, with Box-Eaves
elsif @gable1type=="Verge by Height" ###
  if not @verge1 or @verge1<=0
    @verge1=4.inch if @model.options["UnitsOptions"]["LengthUnit"]<2
    @verge1=100.mm if @model.options["UnitsOptions"]["LengthUnit"]>1
  end#if
  maxht=(opp+fac_up-10.mm).to_l; maxht_s=maxht.to_s
  prompts=["First Gable Verge Height (#@units): "]
  title="First Gable Verge Height: min. > 0, max. < "+maxht_s+"#@units"
  values=[@verge1]
  @verge1,=inputbox(prompts,values,[""],title)
  @verge1=maxht if not @verge1
  while @verge1>maxht+10.mm || @verge1<=0
    @verge1=maxht.to_l
    @verge1,=inputbox(prompts,[@verge1],[""],title)
	@verge1=maxht if not @verge1
  end#while
  ### do lines
  if not @fstype=~/^Sloping+/
    if @verge1<fac_up###
	  add=fac_up-@verge1###
	  ln1=roof_entities.add_line([ge1.x,ge1.y,ge1.z+add],[cp1.x,cp1.y,cp1.z+opp-@verge1+fac_up])####
      roof_entities.add_line([gs1.x,gs1.y,gs1.z+add],[cp1.x,cp1.y,cp1.z+opp-@verge1+fac_up])####
	else
	  diff=@verge1-fac_up###
	  add=diff/Math::tan(@pitch_r)
	  ge1=ge1.offset(gedge1.line[1].reverse,add)
	  gs1=gs1.offset(gedge1.line[1],add)
	  ln1=roof_entities.add_line([ge1.x,ge1.y,ge1.z],[cp1.x,cp1.y,cp1.z+opp-@verge1+fac_up])####
      roof_entities.add_line([gs1.x,gs1.y,gs1.z],[cp1.x,cp1.y,cp1.z+opp-@verge1+fac_up])####
	end#if
  else### sloping fascia
    if @verge1<fac_up###
	  add=fac_up-@verge1###
	  sf_out=add*Math::tan(@pitch_r)
	  sf_up =sf_out*Math::tan(@pitch_r)
	  gs1=gs1.offset(gs1.vector_to(ge1),fac_out-sf_out)
	  ge1=ge1.offset(ge1.vector_to(gs1),fac_out-sf_out)
	  ln1=roof_entities.add_line([ge1.x,ge1.y,ge1.z+add],[cp1.x,cp1.y,cp1.z+opp-@verge1+fac_up+sf_up])####
      roof_entities.add_line([gs1.x,gs1.y,gs1.z+add],[cp1.x,cp1.y,cp1.z+opp-@verge1+fac_up+sf_up])####
	else
	  diff=@verge1-fac_up###
	  add=diff/Math::tan(@pitch_r)
	  ge1=ge1.offset(gedge1.line[1].reverse,add)
	  gs1=gs1.offset(gedge1.line[1],add)
	  ln1=roof_entities.add_line([ge1.x,ge1.y,ge1.z],[cp1.x,cp1.y,cp1.z+opp-@verge1+fac_up])####
      roof_entities.add_line([gs1.x,gs1.y,gs1.z],[cp1.x,cp1.y,cp1.z+opp-@verge1+fac_up])####
	end#if
  end#if
  gab1=nil; ln1.faces.each{|e|gab1=e if e.bounds.max.z!=roof_group.bounds.max.z}
  gab1.pushpull(-@gable1,false) if gab1
  ###
elsif @gable1type=="Verge by Width"###
  if not @verge1w or @verge1w<=0
    @verge1w=4.inch if @model.options["UnitsOptions"]["LengthUnit"]<2
    @verge1w=100.mm if @model.options["UnitsOptions"]["LengthUnit"]>1
  end#if
  maxht=((opp+fac_up+10.mm)*Math::cos(@pitch_r)).to_l; maxht_s=maxht.to_s
  prompts=["First Gable Verge Perpendicular Width (#@units): "]
  title="First Gable Verge: min. > 0, max. < "+maxht_s+"#@units"
  values=[@verge1w]
  @verge1w,=inputbox(prompts,values,[""],title)
  @verge1w=maxht if not @verge1w
  while @verge1w>maxht+10.mm || @verge1w<=0
    @verge1w=maxht.to_l
    @verge1w,=inputbox(prompts,[@verge1],[""],title)
	@verge1w=maxht if not @verge1w
  end#while
  ### do lines
  @vert=@verge1w/Math::cos(@pitch_r)###
  ### do lines
  if not @fstype=~/^Sloping+/
    if @vert<fac_up###
	  add=fac_up-@vert###
	  ln1=roof_entities.add_line([ge1.x,ge1.y,ge1.z+add],[cp1.x,cp1.y,cp1.z+opp-@vert])
      roof_entities.add_line([gs1.x,gs1.y,gs1.z+add],[cp1.x,cp1.y,cp1.z+opp-@vert])
	else
	  diff=@vert-fac_up###
	  add=diff/Math::tan(@pitch_r)
	  ge1=ge1.offset(gedge1.line[1].reverse,add)
	  gs1=gs1.offset(gedge1.line[1],add)
	  ln1=roof_entities.add_line([ge1.x,ge1.y,ge1.z],[cp1.x,cp1.y,cp1.z+opp-@vert+fac_up])
      roof_entities.add_line([gs1.x,gs1.y,gs1.z],[cp1.x,cp1.y,cp1.z+opp-@vert+fac_up])
	end#if
  else### sloping fascia
    if @vert<fac_up###
	  add=fac_up-@vert###
	  sf_out=add*Math::tan(@pitch_r)
	  sf_up =sf_out*Math::tan(@pitch_r)
	  gs1=gs1.offset(gs1.vector_to(ge1),fac_out-sf_out)
	  ge1=ge1.offset(ge1.vector_to(gs1),fac_out-sf_out)
	  ln1=roof_entities.add_line([ge1.x,ge1.y,ge1.z+add],[cp1.x,cp1.y,cp1.z+opp+add+sf_up])
      roof_entities.add_line([gs1.x,gs1.y,gs1.z+add],[cp1.x,cp1.y,cp1.z+opp+add+sf_up])
	else
	  diff=@vert-fac_up###
	  add=diff/Math::tan(@pitch_r)
	  ge1=ge1.offset(gedge1.line[1].reverse,add)
	  gs1=gs1.offset(gedge1.line[1],add)
	  up=(cp1.distance(gs1))*Math::tan(@pitch_r)
	  ln1=roof_entities.add_line([ge1.x,ge1.y,ge1.z],[cp1.x,cp1.y,cp1.z+up])
      roof_entities.add_line([gs1.x,gs1.y,gs1.z],[cp1.x,cp1.y,cp1.z+up])
	end#if
  end#if
  gab1=nil; ln1.faces.each{|e|gab1=e if e.bounds.max.z!=roof_group.bounds.max.z}
  gab1.pushpull(-@gable1,false) if gab1
  ### #####################
elsif @gable1type=="Verge by Height, with Fascia"
  if not @verge1 or @verge1<=0
    @verge1=4.inch if @model.options["UnitsOptions"]["LengthUnit"]<2
    @verge1=100.mm if @model.options["UnitsOptions"]["LengthUnit"]>1
  end#if
  maxht=(opp+fac_up-10.mm).to_l; maxht_s=maxht.to_s
  prompts=["First Gable Verge Height (#@units): "]
  title="First Gable Verge Height: min. > 0, max. < "+maxht_s+"#@units"
  values=[@verge1]
  @verge1,=inputbox(prompts,values,[""],title)
  @verge1=maxht if not @verge1
  while @verge1>maxht+10.mm || @verge1<=0
    @verge1=maxht.to_l
    @verge1,=inputbox(prompts,[@verge1],[""],title)
	@verge1=maxht if not @verge1
  end#while
  ### do lines...
  if not @fstype=~/^Sloping+/
    @inset=@verge1/Math::tan(@pitch_r)
    ge1=ge1.offset(gedge1.line[1].reverse,@inset)
    gs1=gs1.offset(gedge1.line[1],@inset)
    lnA=roof_entities.add_line([ge1.x,ge1.y,ge1.z+fac+fac_out],[cp1.x,cp1.y,cp1.z+opp+fac+fac_out-@verge1])
    lnB=roof_entities.add_line([cp1.x,cp1.y,cp1.z+opp+fac+fac_out-@verge1],[gs1.x,gs1.y,gs1.z+fac+fac_out])
    lnC=roof_entities.add_line([gs1.x,gs1.y,gs1.z+fac+fac_out],[ge1.x,ge1.y,ge1.z+fac+fac_out])
    gab1=roof_entities.add_face(lnA,lnB,lnC)
    gab1.pushpull(-@gable1,false) if gab1
  else#sloping fascia
    sf_in=@verge1/Math::tan(@pitch_r)
    ge1=ge1.offset(ge1.vector_to(gs1),sf_in)
    gs1=gs1.offset(gs1.vector_to(ge1),sf_in)
    lnA=roof_entities.add_line([ge1.x,ge1.y,ge1.z+fac_up],[cp1.x,cp1.y,cp1.z+opp+fac_up+sf_up-@verge1])
    lnB=roof_entities.add_line([cp1.x,cp1.y,cp1.z+opp+fac_up+sf_up-@verge1],[gs1.x,gs1.y,gs1.z+fac_up])
    lnC=roof_entities.add_line([gs1.x,gs1.y,gs1.z+fac_up],[ge1.x,ge1.y,ge1.z+fac_up])
    gab1=roof_entities.add_face(lnA,lnB,lnC)
    gab1.pushpull(-@gable1,false) if gab1
  end#if
  ###
elsif @gable1type=="Verge by Width, with Fascia"
  @verge1w=@fascia if ! @verge1w
  if @verge1w==0
    @verge1w=4.inch if @model.options["UnitsOptions"]["LengthUnit"]<2
    @verge1w=100.mm if @model.options["UnitsOptions"]["LengthUnit"]>1
  end#if
  @vert=@verge1w/Math::cos(@pitch_r)
  @rise=Math::tan(@pitch_r)*@soffit
  maxht=((@rise+@fascia)*Math::cos(@pitch_r)).to_l;maxht_s=maxht.to_s
  prompts=["First Gable Verge Perpendicular Width (#@units): "]
  title="First Gable Verge: min. > 0, max. < "+maxht_s+"#@units"
  values=[@verge1w]
  @verge1w,=inputbox(prompts,values,[""],title)
  @verge1w=@fascia if ! @verge1w
  while @verge1w>maxht || @verge1w<=0
    @verge1w=maxht
    @verge1w,=inputbox(prompts,[@verge1w],[""],title)
  end#while
  ### do lines
  if not @fstype=~/^Sloping+/
    @inset=@vert/Math::tan(@pitch_r)
    ge1=ge1.offset(gedge1.line[1].reverse,@inset)
    gs1=gs1.offset(gedge1.line[1],@inset)
    lnA=roof_entities.add_line([ge1.x,ge1.y,ge1.z+fac+fac_out],[cp1.x,cp1.y,cp1.z+opp+fac+fac_out-@vert])
    lnB=roof_entities.add_line([cp1.x,cp1.y,cp1.z+opp+fac+fac_out-@vert],[gs1.x,gs1.y,gs1.z+fac+fac_out])
    lnC=roof_entities.add_line([gs1.x,gs1.y,gs1.z+fac+fac_out],[ge1.x,ge1.y,ge1.z+fac+fac_out])
    gab1=roof_entities.add_face(lnA,lnB,lnC)
    gab1.pushpull(-@gable1,false) if gab1
  else#sloping fascia ?????????????????????????
    sf_in=@vert/Math::tan(@pitch_r)
    ge1=ge1.offset(ge1.vector_to(gs1),sf_in)
    gs1=gs1.offset(gs1.vector_to(ge1),sf_in)
    lnA=roof_entities.add_line([ge1.x,ge1.y,ge1.z+fac_up],[cp1.x,cp1.y,cp1.z+opp+fac_up+sf_up-@vert])
    lnB=roof_entities.add_line([cp1.x,cp1.y,cp1.z+opp+fac_up+sf_up-@vert],[gs1.x,gs1.y,gs1.z+fac_up])
    lnC=roof_entities.add_line([gs1.x,gs1.y,gs1.z+fac_up],[ge1.x,ge1.y,ge1.z+fac_up])
    gab1=roof_entities.add_face(lnA,lnB,lnC)
    gab1.pushpull(-@gable1,false) if gab1
  end#if
  ###
elsif @gable1type=="Verge with Fascia [All Equal]"###
  @vert=fac_up
  if not @fstype=~/^Sloping+/
    @inset=@vert/Math::tan(@pitch_r)
    ge1=ge1.offset(gedge1.line[1].reverse,@inset)
    gs1=gs1.offset(gedge1.line[1],@inset)
    lnA=roof_entities.add_line([ge1.x,ge1.y,ge1.z+fac+fac_out],[cp1.x,cp1.y,cp1.z+opp+fac+fac_out-@vert])
    lnB=roof_entities.add_line([cp1.x,cp1.y,cp1.z+opp+fac+fac_out-@vert],[gs1.x,gs1.y,gs1.z+fac+fac_out])
    lnC=roof_entities.add_line([gs1.x,gs1.y,gs1.z+fac+fac_out],[ge1.x,ge1.y,ge1.z+fac+fac_out])
    gab1=roof_entities.add_face(lnA,lnB,lnC)
    gab1.pushpull(-@gable1,false) if gab1
  else#sloping fascia ?????????????????????????
    sf_in=@vert/Math::tan(@pitch_r)
    ge1=ge1.offset(ge1.vector_to(gs1),sf_in)
    gs1=gs1.offset(gs1.vector_to(ge1),sf_in)
    lnA=roof_entities.add_line([ge1.x,ge1.y,ge1.z+fac_up],[cp1.x,cp1.y,cp1.z+opp+fac_up+sf_up-@vert])
    lnB=roof_entities.add_line([cp1.x,cp1.y,cp1.z+opp+fac_up+sf_up-@vert],[gs1.x,gs1.y,gs1.z+fac_up])
    lnC=roof_entities.add_line([gs1.x,gs1.y,gs1.z+fac_up],[ge1.x,ge1.y,ge1.z+fac_up])
    gab1=roof_entities.add_face(lnA,lnB,lnC)
    gab1.pushpull(-@gable1,false) if gab1
  end#if
  ###
elsif @gable1type=="Verge by Height, with Box-Eaves"
  @verge1=@fascia if ! @verge1
  if @verge1==0
    @verge1=4.inch if @model.options["UnitsOptions"]["LengthUnit"]<2
    @verge1=100.mm if @model.options["UnitsOptions"]["LengthUnit"]>1
  end#if
  maxht=(opp+@fascia).to_l;maxht_s=maxht.to_s
  prompts=["First Gable Verge Height (#@units): "]
  title="First Gable Verge Height: min. > 0, max. < "+maxht_s+"#@units"
  values=[@verge1]
  @verge1,=inputbox(prompts,values,[""],title)
  @verge1=@fascia if ! @verge1
  while @verge1>maxht || @verge1<=0
    @verge1=maxht
    @verge1,=inputbox(prompts,[@verge1],[""],title)
  end#while
  ### do lines...
  ### get @rise on eaves over soffit at angle
  @rise=Math::tan(@pitch_r)*@soffit
  ge1=ge1.offset(gedge1.line[1].reverse,@soffit)
  gs1=gs1.offset(gedge1.line[1],@soffit)
  roof_entities.add_line(ge1,[ge1.x,ge1.y,ge1.z+@rise+fac+fac_out-@verge1])
  ln1=roof_entities.add_line([ge1.x,ge1.y,ge1.z+@rise+fac+fac_out-@verge1],[cp1.x,cp1.y,cp1.z+opp+fac+fac_out-@verge1])
  roof_entities.add_line([cp1.x,cp1.y,cp1.z+opp+fac+fac_out-@verge1],[gs1.x,gs1.y,gs1.z+@rise+fac+fac_out-@verge1])
  roof_entities.add_line([gs1.x,gs1.y,gs1.z+@rise+fac+fac_out-@verge1],gs1)
  gab1=nil; ln1.faces.each{|e|
    gab1=e if e.bounds.max.z!=roof_group.bounds.max.z
  }
  gab1.pushpull(-@gable1,false) if gab1
  ###
elsif @gable1type=="Verge by Width, with Box-Eaves"
  @verge1w=@fascia if ! @verge1w
  if @verge1w==0
    @verge1w=4.inch if @model.options["UnitsOptions"]["LengthUnit"]<2
    @verge1w=100.mm if @model.options["UnitsOptions"]["LengthUnit"]>1
  end#if
  vert=@verge1w/Math::cos(@pitch_r)
  @rise=Math::tan(@pitch_r)*@soffit
  maxht=((@rise+@fascia)*Math::cos(@pitch_r)).to_l;maxht_s=maxht.to_s
  prompts=["First Gable Verge Perpendicular Width (#@units): "]
  title="First Gable Verge: min. > 0, max. < "+maxht_s+"#@units"
  values=[@verge1w]
  @verge1w,=inputbox(prompts,values,[""],title)
  @verge1w=@fascia if ! @verge1w
  while @verge1w>maxht || @verge1w<=0
    @verge1w=maxht
    @verge1w,=inputbox(prompts,[@verge1w],[""],title)
  end#while
  ### do lines
  @vert=@verge1w/Math::cos(@pitch_r)
  ge1=ge1.offset(gedge1.line[1].reverse,sof)
  gs1=gs1.offset(gedge1.line[1],sof)
  roof_entities.add_line(ge1,[ge1.x,ge1.y,ge1.z+@rise+fac+fac_out-@vert])
  ln1=roof_entities.add_line([ge1.x,ge1.y,ge1.z+@rise+fac+fac_out-@vert],[cp1.x,cp1.y,cp1.z+opp+fac+fac_out-@vert])
  roof_entities.add_line([cp1.x,cp1.y,cp1.z+opp+fac+fac_out-@vert],[gs1.x,gs1.y,gs1.z+@rise+fac+fac_out-@vert])
  roof_entities.add_line([gs1.x,gs1.y,gs1.z+@rise+fac+fac_out-@vert],gs1)
  gab1=nil; ln1.faces.each{|e|
    gab1=e if e.valid? and e.bounds.max.z!=roof_group.bounds.max.z
  }
  gab1.pushpull(-@gable1,false) if gab1
end#if
############## NOW for gable TWO #############----------------------------
if @gable2type=="Verge from Fascia"
  if @fstype=~/^Sloping+/
    gs2=gs2.offset(gs2.vector_to(ge2),fac_out)
    ge2=ge2.offset(ge2.vector_to(gs2),fac_out)
  end#if
  ln1=roof_entities.add_line(ge2,[cp2.x,cp2.y,cp2.z+opp])
  roof_entities.add_line([cp2.x,cp2.y,cp2.z+opp],gs2)
  gab2=nil; ln1.faces.each{|e|
    gab2=e if e.valid? and e.bounds.max.z!=roof_group.bounds.max.z
  }
  gab2.pushpull(-@gable1,false) if gab2
elsif @gable2type=="Verge from Soffit"
  if @fstype=~/^Sloping+/
    gs2=gs2.offset(gs2.vector_to(ge2),fac_out)
    ge2=ge2.offset(ge2.vector_to(gs2),fac_out)
  end#if
  rise=Math::tan(@pitch_r)*sof###
  ge2=ge2.offset(ge2.vector_to(gs2),sof)
  gs2=gs2.offset(gs2.vector_to(ge2),sof)
  ln1=roof_entities.add_line(ge2,[cp2.x,cp2.y,cp2.z+opp-rise])###
  roof_entities.add_line([cp2.x,cp2.y,cp2.z+opp-rise],gs2)###
  gab2=nil; ln1.faces.each{|e|
    gab2=e if e.valid? and e.bounds.max.z!=roof_group.bounds.max.z
  }
  gab2.pushpull(-@gable1,false) if gab2
  ###Verge by Height|Verge by Width|Verge by Height, with Fascia|Verge by Width, with Fascia|Verge by Height, with Box-Eaves|Verge by Width, with Box-Eaves
elsif @gable2type=="Verge by Height" ###
  if not @verge2 or @verge2<=0
    @verge2=4.inch if @model.options["UnitsOptions"]["LengthUnit"]<2
    @verge2=100.mm if @model.options["UnitsOptions"]["LengthUnit"]>1
  end#if
  maxht=(opp+fac_up-10.mm).to_l; maxht_s=maxht.to_s
  prompts=["Second Gable Verge Height (#@units): "]
  title="Second Gable Verge Height: min. > 0, max. < "+maxht_s+"#@units"
  values=[@verge2]
  @verge2,=inputbox(prompts,values,[""],title)
  @verge2=maxht if not @verge2
  while @verge2>maxht+10.mm || @verge2<=0
    @verge2=maxht.to_l
    @verge2,=inputbox(prompts,[@verge2],[""],title)
	@verge2=maxht if not @verge2
  end#while
  ### do lines
  if not @fstype=~/^Sloping+/
    if @verge2<fac_up###
	  add=fac_up-@verge2###
	  ln1=roof_entities.add_line([ge2.x,ge2.y,ge2.z+add],[cp2.x,cp2.y,cp2.z+opp-@verge2+fac_up])####
      roof_entities.add_line([gs2.x,gs2.y,gs2.z+add],[cp2.x,cp2.y,cp2.z+opp-@verge2+fac_up])####
	else
	  diff=@verge2-fac_up###
	  add=diff/Math::tan(@pitch_r)
	  ge2=ge2.offset(gedge2.line[1].reverse,add)
	  gs2=gs2.offset(gedge2.line[1],add)
	  ln1=roof_entities.add_line([ge2.x,ge2.y,ge2.z],[cp2.x,cp2.y,cp2.z+opp-@verge2+fac_up])####
      roof_entities.add_line([gs2.x,gs2.y,gs2.z],[cp2.x,cp2.y,cp2.z+opp-@verge2+fac_up])####
	end#if
  else### sloping fascia
    if @verge2<fac_up###
	  add=fac_up-@verge2###
	  sf_out=add*Math::tan(@pitch_r)
	  sf_up =sf_out*Math::tan(@pitch_r)
	  gs2=gs2.offset(gs2.vector_to(ge2),fac_out-sf_out)
	  ge2=ge2.offset(ge2.vector_to(gs2),fac_out-sf_out)
	  ln1=roof_entities.add_line([ge2.x,ge2.y,ge2.z+add],[cp2.x,cp2.y,cp2.z+opp-@verge2+fac_up+sf_up])####
      roof_entities.add_line([gs2.x,gs2.y,gs2.z+add],[cp2.x,cp2.y,cp2.z+opp-@verge2+fac_up+sf_up])####
	else
	  diff=@verge2-fac_up###
	  add=diff/Math::tan(@pitch_r)
	  ge2=ge2.offset(gedge2.line[1].reverse,add)
	  gs2=gs2.offset(gedge2.line[1],add)
	  ln1=roof_entities.add_line([ge2.x,ge2.y,ge2.z],[cp2.x,cp2.y,cp2.z+opp-@verge2+fac_up])####
      roof_entities.add_line([gs2.x,gs2.y,gs2.z],[cp2.x,cp2.y,cp2.z+opp-@verge2+fac_up])####
	end#if
  end#if
  gab2=nil; ln1.faces.each{|e|gab2=e if e.bounds.max.z!=roof_group.bounds.max.z}
  gab2.pushpull(-@gable1,false) if gab2
  ###
elsif @gable2type=="Verge by Width"###
  if not @verge2w or @verge2w<=0
    @verge2w=4.inch if @model.options["UnitsOptions"]["LengthUnit"]<2
    @verge2w=100.mm if @model.options["UnitsOptions"]["LengthUnit"]>1
  end#if
  maxht=((opp+fac_up+10.mm)*Math::cos(@pitch_r)).to_l; maxht_s=maxht.to_s
  prompts=["Second Gable Verge Perpendicular Width (#@units): "]
  title="Second Gable Verge: min. > 0, max. < "+maxht_s+"#@units"
  values=[@verge2w]
  @verge2w,=inputbox(prompts,values,[""],title)
  @verge2w=maxht if not @verge2w
  while @verge2w>maxht+10.mm || @verge2w<=0
    @verge2w=maxht.to_l
    @verge2w,=inputbox(prompts,[@verge2],[""],title)
	@verge2w=maxht if not @verge2w
  end#while
  ### do lines
  @vert=@verge2w/Math::cos(@pitch_r)###
  ### do lines
  if not @fstype=~/^Sloping+/
    if @vert<fac_up###
	  add=fac_up-@vert###
	  ln1=roof_entities.add_line([ge2.x,ge2.y,ge2.z+add],[cp2.x,cp2.y,cp2.z+opp-@vert])
      roof_entities.add_line([gs2.x,gs2.y,gs2.z+add],[cp2.x,cp2.y,cp2.z+opp-@vert])
	else
	  diff=@vert-fac_up###
	  add=diff/Math::tan(@pitch_r)
	  ge2=ge2.offset(gedge2.line[1].reverse,add)
	  gs2=gs2.offset(gedge2.line[1],add)
	  ln1=roof_entities.add_line([ge2.x,ge2.y,ge2.z],[cp2.x,cp2.y,cp2.z+opp-@vert+fac_up])
      roof_entities.add_line([gs2.x,gs2.y,gs2.z],[cp2.x,cp2.y,cp2.z+opp-@vert+fac_up])
	end#if
  else### sloping fascia
    if @vert<fac_up###
	  add=fac_up-@vert###
	  sf_out=add*Math::tan(@pitch_r)
	  sf_up =sf_out*Math::tan(@pitch_r)
	  gs2=gs2.offset(gs2.vector_to(ge2),fac_out-sf_out)
	  ge2=ge2.offset(ge2.vector_to(gs2),fac_out-sf_out)
	  ln1=roof_entities.add_line([ge2.x,ge2.y,ge2.z+add],[cp2.x,cp2.y,cp2.z+opp+add+sf_up])
      roof_entities.add_line([gs2.x,gs2.y,gs2.z+add],[cp2.x,cp2.y,cp2.z+opp+add+sf_up])
	else
	  diff=@vert-fac_up###
	  add=diff/Math::tan(@pitch_r)
	  ge2=ge2.offset(gedge2.line[1].reverse,add)
	  gs2=gs2.offset(gedge2.line[1],add)
	  up=(cp2.distance(gs2))*Math::tan(@pitch_r)
	  ln1=roof_entities.add_line([ge2.x,ge2.y,ge2.z],[cp2.x,cp2.y,cp2.z+up])
      roof_entities.add_line([gs2.x,gs2.y,gs2.z],[cp2.x,cp2.y,cp2.z+up])
	end#if
  end#if
  gab2=nil; ln1.faces.each{|e|gab2=e if e.bounds.max.z!=roof_group.bounds.max.z}
  gab2.pushpull(-@gable1,false) if gab2
  ### #####################
elsif @gable2type=="Verge by Height, with Fascia"
  if not @verge2 or @verge2<=0
    @verge2=4.inch if @model.options["UnitsOptions"]["LengthUnit"]<2
    @verge2=100.mm if @model.options["UnitsOptions"]["LengthUnit"]>1
  end#if
  maxht=(opp+fac_up-10.mm).to_l; maxht_s=maxht.to_s
  prompts=["Second Gable Verge Height (#@units): "]
  title="Second Gable Verge Height: min. > 0, max. < "+maxht_s+"#@units"
  values=[@verge2]
  @verge2,=inputbox(prompts,values,[""],title)
  @verge2=maxht if not @verge2
  while @verge2>maxht+10.mm || @verge2<=0
    @verge2=maxht.to_l
    @verge2,=inputbox(prompts,[@verge2],[""],title)
	@verge2=maxht if not @verge2
  end#while
  ### do lines...
  if not @fstype=~/^Sloping+/
    @inset=@verge2/Math::tan(@pitch_r)
    ge2=ge2.offset(gedge2.line[1].reverse,@inset)
    gs2=gs2.offset(gedge2.line[1],@inset)
    lnA=roof_entities.add_line([ge2.x,ge2.y,ge2.z+fac+fac_out],[cp2.x,cp2.y,cp2.z+opp+fac+fac_out-@verge2])
    lnB=roof_entities.add_line([cp2.x,cp2.y,cp2.z+opp+fac+fac_out-@verge2],[gs2.x,gs2.y,gs2.z+fac+fac_out])
    lnC=roof_entities.add_line([gs2.x,gs2.y,gs2.z+fac+fac_out],[ge2.x,ge2.y,ge2.z+fac+fac_out])
    gab2=roof_entities.add_face(lnA,lnB,lnC)
    gab2.pushpull(-@gable1,false) if gab2
  else#sloping fascia
    sf_in=@verge2/Math::tan(@pitch_r)
    ge2=ge2.offset(ge2.vector_to(gs2),sf_in)
    gs2=gs2.offset(gs2.vector_to(ge2),sf_in)
    lnA=roof_entities.add_line([ge2.x,ge2.y,ge2.z+fac_up],[cp2.x,cp2.y,cp2.z+opp+fac_up+sf_up-@verge2])
    lnB=roof_entities.add_line([cp2.x,cp2.y,cp2.z+opp+fac_up+sf_up-@verge2],[gs2.x,gs2.y,gs2.z+fac_up])
    lnC=roof_entities.add_line([gs2.x,gs2.y,gs2.z+fac_up],[ge2.x,ge2.y,ge2.z+fac_up])
    gab2=roof_entities.add_face(lnA,lnB,lnC)
    gab2.pushpull(-@gable1,false) if gab2
  end#if
  ###
elsif @gable2type=="Verge by Width, with Fascia"
  @verge2w=@fascia if ! @verge2w
  if @verge2w==0
    @verge2w=4.inch if @model.options["UnitsOptions"]["LengthUnit"]<2
    @verge2w=100.mm if @model.options["UnitsOptions"]["LengthUnit"]>1
  end#if
  @vert=@verge2w/Math::cos(@pitch_r)
  @rise=Math::tan(@pitch_r)*@soffit
  maxht=((@rise+@fascia)*Math::cos(@pitch_r)).to_l;maxht_s=maxht.to_s
  prompts=["Second Gable Verge Perpendicular Width (#@units): "]
  title="Second Gable Verge: min. > 0, max. < "+maxht_s+"#@units"
  values=[@verge2w]
  @verge2w,=inputbox(prompts,values,[""],title)
  @verge2w=@fascia if ! @verge2w
  while @verge2w>maxht || @verge2w<=0
    @verge2w=maxht
    @verge2w,=inputbox(prompts,[@verge2w],[""],title)
  end#while
  ### do lines
  if not @fstype=~/^Sloping+/
    @inset=@vert/Math::tan(@pitch_r)
    ge2=ge2.offset(gedge2.line[1].reverse,@inset)
    gs2=gs2.offset(gedge2.line[1],@inset)
    lnA=roof_entities.add_line([ge2.x,ge2.y,ge2.z+fac+fac_out],[cp2.x,cp2.y,cp2.z+opp+fac+fac_out-@vert])
    lnB=roof_entities.add_line([cp2.x,cp2.y,cp2.z+opp+fac+fac_out-@vert],[gs2.x,gs2.y,gs2.z+fac+fac_out])
    lnC=roof_entities.add_line([gs2.x,gs2.y,gs2.z+fac+fac_out],[ge2.x,ge2.y,ge2.z+fac+fac_out])
    gab2=roof_entities.add_face(lnA,lnB,lnC)
    gab2.pushpull(-@gable1,false) if gab2
  else#sloping fascia ?????????????????????????
    sf_in=@vert/Math::tan(@pitch_r)
    ge2=ge2.offset(ge2.vector_to(gs2),sf_in)
    gs2=gs2.offset(gs2.vector_to(ge2),sf_in)
    lnA=roof_entities.add_line([ge2.x,ge2.y,ge2.z+fac_up],[cp2.x,cp2.y,cp2.z+opp+fac_up+sf_up-@vert])
    lnB=roof_entities.add_line([cp2.x,cp2.y,cp2.z+opp+fac_up+sf_up-@vert],[gs2.x,gs2.y,gs2.z+fac_up])
    lnC=roof_entities.add_line([gs2.x,gs2.y,gs2.z+fac_up],[ge2.x,ge2.y,ge2.z+fac_up])
    gab2=roof_entities.add_face(lnA,lnB,lnC)
    gab2.pushpull(-@gable1,false) if gab2
  end#if
  ###
elsif @gable2type=="Verge with Fascia [All Equal]"###
  @vert=fac_up
  if not @fstype=~/^Sloping+/
    @inset=@vert/Math::tan(@pitch_r)
    ge2=ge2.offset(gedge2.line[1].reverse,@inset)
    gs2=gs2.offset(gedge2.line[1],@inset)
    lnA=roof_entities.add_line([ge2.x,ge2.y,ge2.z+fac+fac_out],[cp2.x,cp2.y,cp2.z+opp+fac+fac_out-@vert])
    lnB=roof_entities.add_line([cp2.x,cp2.y,cp2.z+opp+fac+fac_out-@vert],[gs2.x,gs2.y,gs2.z+fac+fac_out])
    lnC=roof_entities.add_line([gs2.x,gs2.y,gs2.z+fac+fac_out],[ge2.x,ge2.y,ge2.z+fac+fac_out])
    gab2=roof_entities.add_face(lnA,lnB,lnC)
    gab2.pushpull(-@gable1,false) if gab2
  else#sloping fascia ?????????????????????????
    sf_in=@vert/Math::tan(@pitch_r)
    ge2=ge2.offset(ge2.vector_to(gs2),sf_in)
    gs2=gs2.offset(gs2.vector_to(ge2),sf_in)
    lnA=roof_entities.add_line([ge2.x,ge2.y,ge2.z+fac_up],[cp2.x,cp2.y,cp2.z+opp+fac_up+sf_up-@vert])
    lnB=roof_entities.add_line([cp2.x,cp2.y,cp2.z+opp+fac_up+sf_up-@vert],[gs2.x,gs2.y,gs2.z+fac_up])
    lnC=roof_entities.add_line([gs2.x,gs2.y,gs2.z+fac_up],[ge2.x,ge2.y,ge2.z+fac_up])
    gab2=roof_entities.add_face(lnA,lnB,lnC)
    gab2.pushpull(-@gable1,false) if gab2
  end#if
  ###
elsif @gable2type=="Verge by Height, with Box-Eaves"
  @verge2=@fascia if ! @verge2
  if @verge2==0
    @verge2=4.inch if @model.options["UnitsOptions"]["LengthUnit"]<2
    @verge2=100.mm if @model.options["UnitsOptions"]["LengthUnit"]>1
  end#if
  maxht=(opp+@fascia).to_l;maxht_s=maxht.to_s
  prompts=["Second Gable Verge Height (#@units): "]
  title="Second Gable Verge Height: min. > 0, max. < "+maxht_s+"#@units"
  values=[@verge2]
  @verge2,=inputbox(prompts,values,[""],title)
  @verge2=@fascia if ! @verge2
  while @verge2>maxht || @verge2<=0
    @verge2=maxht
    @verge2,=inputbox(prompts,[@verge2],[""],title)
  end#while
  ### do lines...
  ### get @rise on eaves over soffit at angle
  @rise=Math::tan(@pitch_r)*@soffit
  ge2=ge2.offset(gedge2.line[1].reverse,@soffit)
  gs2=gs2.offset(gedge2.line[1],@soffit)
  roof_entities.add_line(ge2,[ge2.x,ge2.y,ge2.z+@rise+fac+fac_out-@verge2])
  ln1=roof_entities.add_line([ge2.x,ge2.y,ge2.z+@rise+fac+fac_out-@verge2],[cp2.x,cp2.y,cp2.z+opp+fac+fac_out-@verge2])
  roof_entities.add_line([cp2.x,cp2.y,cp2.z+opp+fac+fac_out-@verge2],[gs2.x,gs2.y,gs2.z+@rise+fac+fac_out-@verge2])
  roof_entities.add_line([gs2.x,gs2.y,gs2.z+@rise+fac+fac_out-@verge2],gs2)
  gab2=nil; ln1.faces.each{|e|
    gab2=e if e.bounds.max.z!=roof_group.bounds.max.z
  }
  gab2.pushpull(-@gable1,false) if gab2
  ###
elsif @gable2type=="Verge by Width, with Box-Eaves"
  @verge2w=@fascia if ! @verge2w
  if @verge2w==0
    @verge2w=4.inch if @model.options["UnitsOptions"]["LengthUnit"]<2
    @verge2w=100.mm if @model.options["UnitsOptions"]["LengthUnit"]>1
  end#if
  vert=@verge2w/Math::cos(@pitch_r)
  @rise=Math::tan(@pitch_r)*@soffit
  maxht=((@rise+@fascia)*Math::cos(@pitch_r)).to_l;maxht_s=maxht.to_s
  prompts=["Second Gable Verge Perpendicular Width (#@units): "]
  title="Second Gable Verge: min. > 0, max. < "+maxht_s+"#@units"
  values=[@verge2w]
  @verge2w,=inputbox(prompts,values,[""],title)
  @verge2w=@fascia if ! @verge2w
  while @verge2w>maxht || @verge2w<=0
    @verge2w=maxht
    @verge2w,=inputbox(prompts,[@verge2w],[""],title)
  end#while
  ### do lines
  @vert=@verge2w/Math::cos(@pitch_r)
  ge2=ge2.offset(gedge2.line[1].reverse,sof)
  gs2=gs2.offset(gedge2.line[1],sof)
  roof_entities.add_line(ge2,[ge2.x,ge2.y,ge2.z+@rise+fac+fac_out-@vert])
  ln1=roof_entities.add_line([ge2.x,ge2.y,ge2.z+@rise+fac+fac_out-@vert],[cp2.x,cp2.y,cp2.z+opp+fac+fac_out-@vert])
  roof_entities.add_line([cp2.x,cp2.y,cp2.z+opp+fac+fac_out-@vert],[gs2.x,gs2.y,gs2.z+@rise+fac+fac_out-@vert])
  roof_entities.add_line([gs2.x,gs2.y,gs2.z+@rise+fac+fac_out-@vert],gs2)
  gab2=nil; ln1.faces.each{|e|
    gab2=e if e.valid? and e.bounds.max.z!=roof_group.bounds.max.z
  }
  gab2.pushpull(-@gable1,false) if gab2
end#if
### end making verges---------------------------------------------------
###v3.0 ###-------------------------------------------------------------
### 1st make hole in base...
iface=roof_entities.add_face(p1,p2,p3,p4)
iverts=iface.vertices
iface.erase! if iface.valid?
### use sof_up if 0 no change ?
iverts.each{|v|
  next if not v.valid?
  pos=v.position
  posn=[pos.x,pos.y,pos.z+sof_up]
  vec=pos.vector_to(posn)
  tran=Geom::Transformation.translation(vec)
  roof_entities.transform_entities(tran,v)
}
### intersect all so that edges form...
rtr=Geom::Transformation.new()
8.times{
  roof_entities.intersect_with(true,rtr,roof_entities,rtr,true,roof_entities.to_a)
}
### erase coplanar edges...
roof_entities.length.times{
 roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Edge
    if e.faces[1] and not e.faces[2]
	  if not e.faces[0].normal.z.abs.is_near_to?(0.0) ### NOT coplanar in gable
	    e.erase! if e.faces[0].normal==e.faces[1].normal and e.valid?
	  end#if
	end#if
	e.erase! if e.valid? and not e.faces[0] ### =faceless
  end#if
 }
}
### move whole roof group down by sof_up so it sits on selected face.
trz=Geom::Transformation.new([0,0,sof_up*-1])
roof_group.move!(trz)
###v3.0 ###-------------------------------------
###
soffits=[];slopes=[];verticals=[]
roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Face
    if e.normal.z.is_near_to?(1.0) and e.bounds.center.z==roof_group.bounds.min.z
      e.reverse!###v3.1
    end#if
    if e.normal.z.is_near_to?(0.0)
	  verticals << e
	else
	  soffits << e if e.normal.z < 0
      slopes << e if e.normal.z > 0
	end#if
  end#if
}
gables=[]###
verticals.each{|e|
  gables << e if e.edges.length==3 || (e.edges.length==5 && e.bounds.max.z-e.bounds.min.z!=fac_up)
  inline3=0
  e.edges.each{|ee|
    if ee.start.position.z.is_near_to?(ee.end.position.z)
			inline3=inline3+1
		end#if
  }
  gables << e if e.edges.length==6 && (inline3==2 && e.bounds.min.z.is_near_to?(roof_group.bounds.min.z))
  gables << e if e.edges.length==7 && (inline3==3 && e.bounds.min.z.is_near_to?(roof_group.bounds.min.z))
  gab=false;horz=[]
  if e.edges.length==4
    verts=e.vertices; vz=verts[0].position.z; q=1
	(verts-[verts[0]]).each{|v| q=q+1 if v.position.z.is_near_to?(vz) }
	gab=true if q!=2
  end#if
  gables << e if gab
}
if gables[0] and gables[0].normal==gable1normal
  gab1=gables[0]
  gab2=gables[1]
elsif gables[0]
  gab1=gables[1]
  gab2=gables[0]
end#if
### v3.1
rayt=@model.raytest(gab1.bounds.center, gab1.normal)
if rayt and rayt[1][-1]==gab2
  gab1.reverse!
end#if
rayt=@model.raytest(gab2.bounds.center, gab2.normal)
if rayt and rayt[1][-1]==gab1
  gab2.reverse!
end#if
### v3.1
fascias=verticals-gables
### sort any box verge bits of fascias that should be soffit...
fascias.each{|f|
  if f.edges.length==4 ### =horiz bit: get horiz edges that's top one...
    topedge=nil
    f.edges.each{|e|
      topedge=e if e.bounds.max.z==e.bounds.min.z && e.bounds.max.z==f.bounds.max.z
    }
    box=false
	topedge.faces.each{|e|box=true if e.normal.z<0}if topedge
    if box
      fascias-[f]
	  soffits << f if not (f.normal.z > 0.99999)
    end#if
  end#if
}
### move lids from slopes
slopes.each{|e|
  if e.normal.z > 0.99999
	slopes-[e]
	fascias << e
  end#if
}
###
###v3.0 we must get sloping fascias into the correct array
### first go through soffits, IF edge common with slope then =fascia.
if fac_out>0
  soffits.each{|s|
    is_fac=false
    s.edges.each{|e|
	  slopes.each{|r|
	    r.edges.each{|ee|
		  is_fac=true if ee==e
		}
	  }
	}
	if is_fac
	  soffits=soffits-[s]
	  fascias << s
	end#if
  }
end#if###v3.0
### sort out layer, materials etc ###-----------------------------------
def Roof::dialog_mats_gable ()
### materials dialog ###------------------------------------------------
  model=Sketchup.active_model
  ### layers...
  mlayers=model.layers
  alayer=model.active_layer
  model.active_layer=nil
  deflayer=model.active_layer.name
  model.active_layer=alayer
  rlayer="-ROOF"### edit this to suit your desired default roof layer.
  layers=[]
  mlayers.each{|layer|layers << layer.name if layer.name!=deflayer}
  layers=[rlayer,deflayer]+layers.sort!
  layers=layers.uniq.join('|')
  ### materials...
  mat_roof="Roof Surface"
  mat_fascia="Roof Fascia"
  mat_soffit="Roof Soffit"
  mat_gable1="<Default>"
  mat_gable2="<Default>"
  materials=model.materials
  mats=[]
  materials.each{|mat|mats << mat.display_name}
  mats.sort!
  list_mat_roof=((["<Default>"]+[mat_roof]+mats).uniq).join("|")
  list_mat_fascia=((["<Default>"]+[mat_fascia]+mats).uniq).join("|")
  list_mat_soffit=((["<Default>"]+[mat_soffit]+mats).uniq).join("|")
  list_mat_gable1=((["<Default>"]+["Roof Gable 1"]+["Roof Gable 2"]+[mat_roof]+mats).uniq).join("|")
  list_mat_gable2=((["<Default>"]+["Roof Gable 1"]+["Roof Gable 2"]+[mat_roof]+mats).uniq).join("|")
  ### set up variables...
  @layer=rlayer if ! @layer
  @mat_roof=mat_roof if ! @mat_roof
  @mat_fascia=mat_fascia if ! @mat_fascia
  @mat_soffit=mat_soffit if ! @mat_soffit
  @mat_gable1=mat_gable1 if ! @mat_gable1
  @mat_gable2=mat_gable2 if ! @mat_gable2
  enums=[layers,list_mat_roof,list_mat_fascia,list_mat_soffit,list_mat_gable1,list_mat_gable2]
  prompts=["Roof Layer:  ","Roof Surface:  ","Roof Fascia:  ","Roof Soffit: ","First Roof Gable: ","Second Roof Gable: "]
  values=[@layer,@mat_roof,@mat_fascia,@mat_soffit,@mat_gable1,@mat_gable2]
  @other_results=(inputbox prompts,values,enums,"Roof Layer and Materials")
  @layer,@mat_roof,@mat_fascia,@mat_soffit,@mat_gable1,@mat_gable2=@other_results if @other_results
  @other="No" if ! @other_results
  ###
end#def
### run material dialog
  Roof::dialog_mats_gable() if @other=="Yes"
###---------------------------------------------------------------------
  #mat_roof="Roof Surface"
  #mat_fascia="Roof Fascia"
  #mat_soffit="Roof Soffit"
  #mat_gable1="<Default>"
  #mat_gable2="<Default>"
if @other=="Yes"
 ###....................................................................
 ##sort for lids
 lids=false; fascias.each{|e|lids=true if e.normal.z==1}
 if lids 
   ### VCB and status info
   Sketchup::set_status_text(("'Lid' Material..." ), SB_PROMPT)
   Sketchup::set_status_text("", SB_VCB_LABEL)
   Sketchup::set_status_text("", SB_VCB_VALUE)
   if UI.messagebox("Are the Gable Fascia 'Lids'\nto be in the Fascia Material ? ",MB_YESNO,"'Lid' Material")==7 ### 6=YES 7=NO
     fascias.each{|e|
       if e.normal.z==1
	     fascias-[e]
	     soffits << e
	   end#if
     }
  end#if
 end#if
  ### note that layer is set after groups combined...
  extg=nil;@model.layers.each{|lyr|extg=lyr if lyr.name==@layer}
  if @layer=="<Default>"
    layr=nil
  else
    if extg
      layr=extg
    else
      layr=@model.layers.add(@layer)
    end#if
  end#if
  ### materials...
  ### roof
  extg=nil;@model.materials.each{|mat|extg=mat if mat.display_name==@mat_roof}
  if @mat_roof=="<Default>"
    mat_r=nil
  else
    if extg
      mat_r=extg
    else
      mat_r=@model.materials.add(@mat_roof)
      mat_r.color=[111,111,111]### adjust default colours here.
    end#if
  end#if
  slopes.each{|e|e.material=mat_r}
  ### fascia
  extg=nil;@model.materials.each{|mat|extg=mat if mat.display_name==@mat_fascia}
  if @mat_fascia=="<Default>"
    mat_f=nil
  else
    if extg
      mat_f=extg
    else
      mat_f=@model.materials.add(@mat_fascia)
      mat_f.color=[66,66,66]### adjust default colours here.
    end#if
  end#if
  fascias.each{|e|e.material=mat_f}
  ### soffit
  extg=nil;@model.materials.each{|mat|extg=mat if mat.display_name==@mat_soffit}
  if @mat_soffit=="<Default>"
    mat_s=nil
  else
    if extg
      mat_s=extg
    else
      mat_s=@model.materials.add(@mat_soffit)
      mat_s.color=[222,222,222]### adjust default colours here.
    end#if
  end#if
  soffits.each{|e|e.material=mat_s}
  ### gable
  extg=nil;@model.materials.each{|mat|extg=mat if mat.display_name==@mat_gable1}
  if @mat_gable1=="<Default>"
    mat_g1=nil
  else
    if extg
      mat_g1=extg
    else
      mat_g1=@model.materials.add(@mat_gable1)
      ### adjust default colours here.
	  mat_g1.color=[234,234,234] if @mat_gable1=="Roof Surface"
      mat_g1.color=[255,255,255] if @mat_gable1=="Roof Gable 1"#white
      mat_g1.color=[255,255,255] if @mat_gable1=="Roof Gable 2"#white
    end#if
  end#if
  gab1.material=mat_g1 if gab1
  extg=nil;@model.materials.each{|mat|extg=mat if mat.display_name==@mat_gable2}
  if @mat_gable2=="<Default>"
    mat_g2=nil
  else
    if extg
      mat_g2=extg
    else
      mat_g2=@model.materials.add(@mat_gable2)
      ### adjust default colours here.
	  mat_g2.color=[234,234,234] if @mat_gable2=="Roof Surface"
      mat_g2.color=[255,255,255] if @mat_gable2=="Roof Gable 1"#white
      mat_g2.color=[255,255,255] if @mat_gable2=="Roof Gable 2"#white
    end#if
  end#if
  gab2.material=mat_g2 if gab2
end#if
### fix for fascia lid et al ###########################################
slopes.each{|e|e.material=mat_r if not e.normal.z==1}
########################################################################
### fix for messy fascia gables < soffit rise
  if (sof*Math::tan(@pitch_r)>fac_up and @fstype=~/Sloping$/) and (@gable1type=~/with Fascia/ or @gable2type=~/with Fascia/)
    dinc=((((((((Math::atan(fac_up/sof)).radians)))*1000).to_i).to_f)/1000).to_s
	rinc="1:"+(((((sof/fac_up)*1000).to_i).to_f)/1000).to_s
	sinc=(((((fac_up/(Math::tan(@pitch_r)))*1000).to_i).to_f)/1000).to_l.to_s
	finc=(((((sof*(Math::tan(@pitch_r)))*1000).to_i).to_f)/1000).to_l.to_s
    UI.messagebox("Roof Gable: Warning.\n\nThere might be some unexpected Gable Geometry.\nThe Gable Fascia Size is less than the Rise on the Sloping Soffit.\n\nDoing ONE of the following might avoid this:\n\n  Make the slope < "+dinc+" degrees or "+rinc+"\n  Make the soffit < "+sinc+@units+"\n  Make the fascia > "+finc+@units+"\n")
  end#if
#########################
  @model.commit_operation
  Sketchup.send_action "selectSelectionTool:"
end#def gable
###---------------------------------------------------------------------







def Roof::pyramid(pts)
  ### check face -------------------------------------------------------
  Sketchup::set_status_text "Roof Pyramid:"
  @model=Sketchup.active_model
	entities=@model.active_entities
	p1=pts[0]; p2=pts[1]; pd=pts[2]
	z1=p1.z; p2.z=z1; pd.z=z1
	if (p1==p2 || p2==pd) || (pd==p1)
	  UI.messagebox("Roof Pyramid:\nCan't have points in the same place in plan !\nTry again...")
	  Sketchup.send_action "selectSelectionTool:"
	  return nil
	end#if
	ang12=(p1.vector_to(p2)).to_flat_angle
	ang2d=(p2.vector_to(pd)).to_flat_angle
	ang=(ang2d-ang12).to_f
	if ang.abs.is_near_to?(0.0.degrees) || ang.abs.is_near_to?(180.0.degrees)
	  UI.messagebox("Roof Pyramid:\nCan't have 3 points in a line !\nTry again...")
	  Sketchup.send_action "selectSelectionTool:"
	  return nil
	end#if
### do gable dialog...
def Roof::dialog_pyramid ()
### dialog ###----------------------------------------------------------
@pitchG="1:2" if ! @pitchG && @model.options["UnitsOptions"]["LengthUnit"]<2
@pitchG="30" if ! @pitchG && @model.options["UnitsOptions"]["LengthUnit"]>1
@fascia=8.inch if ! @fascia && @model.options["UnitsOptions"]["LengthUnit"]<2
@fascia=200.mm if ! @fascia && @model.options["UnitsOptions"]["LengthUnit"]>1
@soffit=16.inch if ! @soffit && @model.options["UnitsOptions"]["LengthUnit"]<2
@soffit=400.mm if ! @soffit && @model.options["UnitsOptions"]["LengthUnit"]>1
@fstype="Vertical+Horizontal" if ! @fstype###v3.0
fstypes="Vertical+Horizontal|Vertical+Sloping|Sloping+Sloping|Sloping+Horizontal"###v3.0
@other="No" if ! @other
@units="inches" if @model.options["UnitsOptions"]["LengthUnit"]==0
@units="feet" if @model.options["UnitsOptions"]["LengthUnit"]==1
@units="mm" if @model.options["UnitsOptions"]["LengthUnit"]==2
@units="cm" if @model.options["UnitsOptions"]["LengthUnit"]==3
@units="m" if @model.options["UnitsOptions"]["LengthUnit"]==4
enums=["","","",fstypes,"Yes|No"]###v3.0
prompts=["Roof Slope (in Degrees or A:B or ^Height(#{@units})):  ","Eaves Fascia Size (#{@units}):  ","Eaves Soffit Size (#{@units}):  ","Fascia+Soffit Type:  ","Set Layer and Materials ? :"]###v3.0
values=[@pitchG,@fascia,@soffit,@fstype,@other]###v3.0
@results=(inputbox prompts,values,enums,"Pyramid Roof Settings")
return nil if ! @results
(@pitch_s,@fascia_,@soffit_,@fstype_,@other_)=@results###v3.0
@pitchG=@pitch_s;@fascia=@fascia_;@soffit=@soffit_;
@fstype=@fstype_;@other=@other_###v3.0
### sort if ^height or slope...
@hi=false
if @pitch_s.split("")[0]=="^"
  @pht=@pitch_s[1..-1].to_f
  @pht=@pht.to_l if @units=="inches"
  @pht=@pht.feet if @units=="feet"
  @pht=@pht.mm if @units=="mm"
  @pht=@pht.cm if @units=="cm"
  @pht=@pht.m if @units=="m"
  @hi=true
  @pitch_d=2
  @pitch_d=0 if @pht <1.mm
else#it's an angle
  ### get angle of roof in radians
  ratio=@pitch_s.split(":").to_f
  if ratio[1]
    tan=ratio[0]/ratio[1]
    @pitch_d=(Math::atan(tan)).radians
  else
    @pitch_d=ratio[0]
  end#if
end#if
@pitch_r=@pitch_d.degrees  ### to radians ###------------------------
if @pitch_d<1
  UI.messagebox("Pyramid Roof:\n\Pitch can't be less than 1 degree.")
  @pitchG="1";@pitch_s="1";@pitch_d=1;@pitch_r=1.degrees
  Roof::dialog_pyramid
  return nil
end#if
if @pitch_d>89 
  UI.messagebox("Pyramid Roof:\n\nPitch can't be more than 89 degrees.")
  @pitchG="89";@pitch_s="89";@pitch_d=89;@pitch_r=89.degrees
  Roof::dialog_pyramid
  return nil
end#if
if @fascia_ <0
  UI.messagebox("Pyramid Roof:\n\nFascia can't be LESS THAN ZERO.\nResetting it to ZERO...")
  @fascia=0.to_l;@fascia_=0.to_l
  Roof::dialog_pyramid
  return nil
end#if
if @soffit_ <0
  UI.messagebox("Pyramid Roof:\n\nEaves can't be LESS THAN ZERO.\nResetting it to ZERO...\n\nFor a 'parapet-eaves' first use 'Offset' on the face to inset its edges ...")
  @soffit=0.to_l;@soffit_=0.to_l
  Roof::dialog_pyramid
  return nil
end#if
###v3.0 ###-----------------------------------------
if @fascia_ ==0 && @fstype_=="Vertical+Sloping"
  UI.messagebox("Pyramid Roof:\n\nIf the Fascia is ZERO then you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Vertical+Horizontal'...")
  @fstype="Vertical+Horizontal"; @fstype_="Vertical+Horizontal"
  Roof::dialog_pyramid
  return nil
end#if
if @fascia_ ==0 && @fstype_=="Sloping+Sloping"
  UI.messagebox("Pyramid Roof:\n\nIf the Fascia is ZERO then you can't have a Sloping Soffit.\nResetting Fascia+Soffit Type to 'Sloping+Horizontal'...")
  @fstype="Sloping+Horizontal"; @fstype_="Sloping+Horizontal"
  Roof::dialog_pyramid
  return nil
end#if
###v3.0
###
end#def dialog
### run dialog
  Roof::dialog_pyramid()
  (Sketchup.send_action "selectSelectionTool:"; return nil) if ! @results
###
Sketchup::set_status_text "Roof Pyramid: Processing..."
###v3.0 sort out sloping fascia and soffit
### defaults=Vertical+Horizontal
fac=@fascia; fac_up=fac; fac_out=0.0
sof=@soffit; sof_up=0.0
if @fstype=~/Sloping\+/
  fac_up =fac*Math::cos(@pitch_r)
  fac_out=fac*Math::sin(@pitch_r)
end#if
if @fstype=~/Sloping$/
  sof_up=sof*Math::tan(@pitch_r)
end#if
###v3.0
###---------------------------------------------------------------------
	cwise=false
	cwise=true if (ang12>=0.0.degrees && ang12<90.0.degrees) && (ang2d<ang12 || ang2d>ang12+180.0.degrees)
	cwise=true if (ang12>=90.0.degrees && ang12<180.0.degrees) && (ang2d>ang12 && ang2d<ang12+180.0.degrees)
	cwise=true if (ang12>=180.0.degrees && ang12<270.0.degrees) && (ang2d<ang12 || ang2d<ang12-180.0.degrees)
	cwise=true if (ang12>=270.0.degrees && ang12<=360.0.degrees) && (ang2d<ang12 && ang2d>ang12-180.0.degrees)
	ang90=90.0.degrees
	#ang90=-ang90 if cwise
	(pt=p1;p1=p2;p2=pt) if cwise
	###
	roof_group=entities.add_group
	roof_entities=roof_group.entities
	ln1=roof_entities.add_line(p1,p2)
	###len=ln1.length ### width of base
    ### get dist of p3d from line of ln1,this is depth of base
	dist=pd.distance_to_line(ln1.line)
	ln1.erase! if ln1.valid?
	### add p3 along vect p2,p1 at dist,rotate p3 90deg
	vec=p2.vector_to(p1)
	p3=p2.offset(vec,dist)
	rotate=Geom::Transformation.rotation(p2,[0,0,1],-ang90)
	p3=p3.transform!(rotate)
	#ln2=roof_entities.add_line(p2,p3)
	###add p4 along vect p1,p2 at dist,rotate p4 -90deg
	vec=p1.vector_to(p2)
	p4=p1.offset(vec,dist)
	rotate=Geom::Transformation.rotation(p1,[0,0,1],ang90)
	p4=p4.transform!(rotate)
	### p1,p2,p3,p4 are corners of base 'hole'
### do Pyramid roof...
@model.start_operation("Pyramid Roof")
### set up roof group etc ###-------------------------------------------
roof_group=entities.add_group
roof_entities=roof_group.entities
iface=roof_entities.add_face(p1,p2,p3,p4)
if @soffit==0
  face=iface
else
  face=iface.offset_eaves(sof)###v3.0
  if not face
    face=iface
	fface=face.offset_eaves(fac_out) if fac_out >0###v3.0
  else
    fface=face.offset_eaves(fac_out) if fac_out >0###v3.0
    iface.edges.each{|e|e.erase! if e.valid?}
  end#if
end#if
epoints=Roof.ordered_edge_points(face)
fepoints=Roof.ordered_edge_points(fface) if fac_out>0###v3.0
roof_entities.to_a.each{|e|e.erase! if e.valid? && e.class==Sketchup::Face}
efpoints=epoints; efpoints=efpoints << efpoints[0]
eedges=roof_entities.add_edges(efpoints)
fefpoints=fepoints if fac_out>0###v3.0
fefpoints=fefpoints << fefpoints[0] if fac_out>0###v3.0
fefpoints=efpoints if fac_out==0###v3.0
feedges=roof_entities.add_edges(fefpoints)###v3.0
### allow for fascia...
fpoints=[]
fefpoints.each{|e|fpoints << [e.x,e.y,e.z+fac_up]}###v3.0
### --------------------------------------------------------------------
### @pitch_d = angle of roof at pitch side = ln1
ln1=roof_entities.add_line(p1,p2)
ctr=Geom::Point3d.linear_combination(0.5,ln1.start.position,0.5,ln1.end.position)
apx=Geom::Point3d.linear_combination(0.5,epoints[0],0.5,epoints[2])
adj=ctr.distance(apx)+@soffit
if @hi ###^nnn height added
  opp=@pht
else
  opp=Math::tan(@pitch_r)*adj
end#if
apex=[apx.x,apx.y,apx.z+opp+fac_up]###v3.0
### add faces to roof ###v2.1
roof_entities.add_face(fpoints[0],fpoints[1],apex)
roof_entities.add_face(fpoints[1],fpoints[2],apex)
roof_entities.add_face(fpoints[2],fpoints[3],apex)
roof_entities.add_face(fpoints[3],fpoints[0],apex)
### add in soffit
soffit=roof_entities.add_face(eedges)###v2.1
soffit.reverse! if soffit.normal.z > 0
### add fascia, if any
if @fascia >0
  for i in 0...efpoints.length-1###
    roof_entities.add_face(efpoints[i],fpoints[i],fpoints[i+1],efpoints[i+1])
  end#for
end#if
verticals=[]
roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Face
    verticals << e if e.normal.z.is_near_to?(0.0) ###v2.1
  end#if
}
###v3.0 remove spare unfaced lines
feedges.each{|e|e.erase!} if fac_out>0
###v3.0
### make hole in base...
iface=roof_entities.add_face(p1,p2,p3,p4)
###
###v3.0
iverts=iface.vertices
###v3.0
iface.erase! if iface.valid?
###v3.0
### use sof_up if 0 no change ?
iverts.each{|v|
  next if not v.valid?
  pos=v.position
  posn=[pos.x,pos.y,pos.z+sof_up]###v3.0
  vec=pos.vector_to(posn)
  tran=Geom::Transformation.translation(vec)
  roof_entities.transform_entities(tran,v)
}
### erase coplanar edges.
roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Edge
    if e.faces[1] and not e.faces[2]
	  e.erase! if e.faces[0].normal==e.faces[1].normal and e.valid?
	end#if
  end#if
}
### move whole roof group down by sof_up so it sits on selected face.
trz=Geom::Transformation.new([0,0,sof_up*-1])
roof_group.move!(trz)
###v3.0
###
### NOW sort out face bits for their materials--------------------------
soffits=[];slopes=[];fascias=[]
roof_entities.to_a.each{|e|
  if e.valid? and e.class==Sketchup::Face
    if e.normal.z.is_near_to?(0.0)###v2.1
      fascias << e
	  ### ~vertical
	else
	  soffits << e if e.normal.z < 0
	  ### downwards soffits
      slopes << e if e.normal.z > 0
	  ### upwards but not lid
    end#if
  end#if
}
###v3.0 we must get sloping fascias into the correct array
### first go through soffits, IF edge common with slope then =fascia.
if fac_out>0
  soffits.each{|s|
    is_fac=false
    s.edges.each{|e|
	  slopes.each{|r|
	    r.edges.each{|ee|
		  is_fac=true if ee==e
		}
	  }
	}
	if is_fac
	  soffits=soffits-[s]
	  fascias << s
	end#if
  }
end#if###v3.0
### sort out layer, materials etc ###-----------------------------------
def Roof::dialog_mats_pyramid ()
### materials dialog ###------------------------------------------------
  model=Sketchup.active_model
  ### layers...
  mlayers=model.layers
  alayer=model.active_layer
  model.active_layer=nil
  deflayer=model.active_layer.name
  model.active_layer=alayer
  rlayer="-ROOF"### edit this to suit your desired default roof layer.
  layers=[]
  mlayers.each{|layer|layers << layer.name if layer.name!=deflayer}
  layers=[rlayer,deflayer]+layers.sort!
  layers=layers.uniq.join('|')
  ### materials...
  mat_roof="Roof Surface"
  mat_fascia="Roof Fascia"
  mat_soffit="Roof Soffit"
  materials=model.materials
  mats=[]
  materials.each{|mat|mats << mat.display_name}
  mats.sort!
  list_mat_roof=((["<Default>"]+[mat_roof]+mats).uniq).join("|")
  list_mat_fascia=((["<Default>"]+[mat_fascia]+mats).uniq).join("|")
  list_mat_soffit=((["<Default>"]+[mat_soffit]+mats).uniq).join("|")
  ### set up variables...
  @layer=rlayer if ! @layer
  @mat_roof=mat_roof if ! @mat_roof
  @mat_fascia=mat_fascia if ! @mat_fascia
  @mat_soffit=mat_soffit if ! @mat_soffit
  enums=[layers,list_mat_roof,list_mat_fascia,list_mat_soffit]
  prompts=["Roof Layer:  ","Roof Surface:  ","Roof Fascia:  ","Roof Soffit: "]
  values=[@layer,@mat_roof,@mat_fascia,@mat_soffit]
  @other_results=(inputbox prompts,values,enums,"Roof Layer and Materials")
  @layer,@mat_roof,@mat_fascia,@mat_soffit=@other_results if @other_results
  @other="No" if ! @other_results
  ###
end#def
### run material dialog
  Roof::dialog_mats_pyramid() if @other=="Yes"
###---------------------------------------------------------------------
  #mat_roof="Roof Surface"
  #mat_fascia="Roof Fascia"
  #mat_soffit="Roof Soffit"
if @other=="Yes"
  ### note that layer is set after groups combined...
  extg=nil;@model.layers.each{|lyr|extg=lyr if lyr.name==@layer}
  if @layer=="<Default>"
    layr=nil
  else
    if extg
      layr=extg
    else
      layr=@model.layers.add(@layer)
    end#if
  end#if
  ### materials...
  ### fascia
  extg=nil;@model.materials.each{|mat|extg=mat if mat.display_name==@mat_fascia}
  if @mat_fascia=="<Default>"
    mat_f=nil
  else
    if extg
      mat_f=extg
    else
      mat_f=@model.materials.add(@mat_fascia)
      mat_f.color=[66,66,66]### adjust default colours here.
    end#if
  end#if
  fascias.each{|e|e.material=mat_f}
  ### soffit
  extg=nil;@model.materials.each{|mat|extg=mat if mat.display_name==@mat_soffit}
  if @mat_soffit=="<Default>"
    mat_s=nil
  else
    if extg
      mat_s=extg
    else
      mat_s=@model.materials.add(@mat_soffit)
      mat_s.color=[222,222,222]### adjust default colours here.
    end#if
  end#if
  soffits.each{|e|e.material=mat_s}
  ### roof
  extg=nil;@model.materials.each{|mat|extg=mat if mat.display_name==@mat_roof}
  if @mat_roof=="<Default>"
    mat_r=nil
  else
    if extg
      mat_r=extg
    else
      mat_r=@model.materials.add(@mat_roof)
      mat_r.color=[111,111,111]### adjust default colours here.
    end#if
  end#if
  slopes.each{|e|e.material=mat_r}
end#if
### tidy up at end ###--------------------------------------------------
@model.active_view.invalidate
@model.commit_operation
Sketchup.send_action"selectSelectionTool:"
###
end#def pyramid
###---------------------------------------------------------------------







###---------------------------------------------------------------------
def Roof::get_slope()
  ### check face -------------------------------------------------------
  @model=Sketchup.active_model
  ss=@model.selection
  faces=[]; ss.each{|e|faces << e if e.class==Sketchup::Face}
  if ! faces[0] || faces[1]
    UI.messagebox("Get Slope:\n\nYou must select ONE FACE !")
    return nil
  end#if
  face=faces[0]
  normal=face.normal
  up=Geom::Vector3d.new(0,0,1)
  angle=normal.angle_between(up)###radians
  angle_d=angle.radians###degrees
  angle_s=sprintf("%.3f",angle_d.to_s)### to 3dp
  a=1; b=1/Math::tan(angle).abs
  (a=0;b=1) if angle_d==0###
  (a=1;b=1) if angle_d>=89.9999###
  ratio=a.to_s+" : "+(sprintf("%.3f",b.to_s))### to 3dp
  a=12*Math::tan(angle).abs; b=12
  (a=0;b=12) if angle_d==0###
  (a=12;b=12) if angle_d>=89.9999###
  ratio12=(sprintf("%.3f",a.to_s))+" : "+b.to_s### to 3dp
  UI.messagebox("Angle = "+angle_s+" degrees\nSlope = "+ratio+"\nSlope = "+ratio12)
  ###  
end#def get_slope
###---------------------------------------------------------------------





def Roof::help
  help=Sketchup.find_support_file("RoofHelp.doc","Plugins/")
  if help
    help="file:///"+help
	UI.openURL(help)
  else
    UI.messagebox("***ERROR***\nDOC Roof Help File NOT found.") 
    help=Sketchup.find_support_file("RoofHelp.mht","Plugins/")
	if help
      help="file:///"+help
	  UI.openURL(help)
	else
	  UI.messagebox("***ERROR***\nMHT Roof Help File NOT found.") 
	  help=Sketchup.find_support_file("RoofHelp.pdf","Plugins/")
	  if help
	    help="file:///"+help
	    UI.openURL(help)
      else
	    UI.messagebox("***ERROR***\nPDF Roof Help File NOT found.")
	  end#if
	end#if
  end#if
end#def help
###---------------------------------------------------------------------

###
end#class Roof##########################################################
###---------------------------------------------------------------------





###---------------------------------------------------------------------
class RoofGable
    def initialize
        ###?
		reset
    end
    def reset
        @pts=[]
        @state=0
        @ip=Sketchup::InputPoint.new
        @ip1=Sketchup::InputPoint.new
        @ip2=Sketchup::InputPoint.new
        @ip3=Sketchup::InputPoint.new
        Sketchup::set_status_text "Roof Gable:  Pick the 1st Corner of the First-Gable [this will also set Z-level]..."
        @drawn=false
		@shift_down_time=0
    end
    def activate
        self.reset
    end
    def deactivate(view)
        view.invalidate if @drawn
        @ip1=nil
        @ip2=nil
        @ip3=nil
    end
    def onMouseMove(flags,x,y,view)
	    view.invalidate
        case @state
        when 0 ### get the first point
            @ip.pick(view,x,y)
            if @ip.valid? && @ip!=@ip1
                @ip1.copy!(@ip)
				@pts[0]=@ip1.position
            end
			view.invalidate
            view.tooltip="1st Corner of First-Gable: "+@ip.tooltip if @ip.valid?
        when 1 ### get the second point
            @ip.pick(view,x,y,@ip1)
            if @ip.valid? && @ip!=@ip2
                @ip2.copy!(@ip)
                @pts[1]=@ip2.position
            end
            view.invalidate
            view.tooltip="2nd Corner of First-Gable: "+@ip.tooltip if @ip.valid?
        when 2 ### get the third point
            @ip.pick(view,x,y,@ip2)
            if @ip.valid? && @ip!=@ip3
                @ip3.copy!(@ip)
                @pts[2]=@ip3.position
            end
            view.invalidate
            view.tooltip="Point on the Line of Second-Gable: "+@ip.tooltip if @ip.valid?
        end
    end
    def onLButtonDown(flags,x,y,view)
        @ip.pick(view,x,y)
        if @ip.valid?
            case @state
            when 0
                if @ip.valid?
                    @pts[0]=@ip.position
                    Sketchup::set_status_text("Roof Gable:  Pick the 2nd Corner of the First-Gable...")
                    @state=1
                end
            when 1
                if @ip.valid? && @ip!=@ip1
                    @pts[1]=@ip.position
                    @state=2
                    Sketchup::set_status_text("Roof Gable:  Pick a Point on the Line of the Second-Gable...")
                end
            when 2
                if @ip.valid? && @ip!=@ip1 && @ip!=@ip2
                    @pts[2]=@ip.position
                    @state=3
                    Roof::gable(@pts)
					return nil
                end
            end
        end
    end
    def onCancel(flag,view)
        view.invalidate if @drawn
        reset
    end
    def draw(view)
        if @ip.valid? && @ip.display?
            @ip.draw(view)
            @drawn=true
        end
        if @state==1
            view.set_color_from_line(@ip1,@ip)
            view.draw(GL_LINE_STRIP,@ip1.position,@ip.position)
            @drawn=true
        elsif @state==2
            view.drawing_color="orange"
            view.draw(GL_LINE_STRIP,@ip1.position,@ip2.position)
            view.set_color_from_line(@ip2,@ip)
            view.draw(GL_LINE_STRIP,@ip2.position,@ip.position)
            @drawn=true
        end
    end
    def onKeyDown(key,repeat,flags,view)
        if key==CONSTRAIN_MODIFIER_KEY && repeat==1
            @shift_down_time=Time.now
            if view.inference_locked?
                view.lock_inference
            elsif @ip.valid?
                view.lock_inference(@ip)
            end
        end
    end
    def onKeyUp(key,repeat,flags,view)
        if key==CONSTRAIN_MODIFIER_KEY && view.inference_locked? && Time.now-@shift_down_time>0.5
            view.lock_inference
        end
        if key==13
            Roof::gable(@pts)
            self.reset
        end
    end
end#class RoofGable



########################################################################
class RoofPyramid
    def initialize
        ###?
		reset
    end
    def reset
        @pts=[]
        @state=0
        @ip=Sketchup::InputPoint.new
        @ip1=Sketchup::InputPoint.new
        @ip2=Sketchup::InputPoint.new
        @ip3=Sketchup::InputPoint.new
        Sketchup::set_status_text "Roof Pyramid:  Pick the 1st Corner of the Pitch-Side [this will also set Z-level]..."
        @drawn=false
		@shift_down_time=0
    end
    def activate
        self.reset
    end
    def deactivate(view)
        view.invalidate if @drawn
        @ip1=nil
        @ip2=nil
        @ip3=nil
    end
    def onMouseMove(flags,x,y,view)
	    view.invalidate
        case @state
        when 0 ### get the first point
            @ip.pick(view,x,y)
            if @ip.valid? && @ip!=@ip1
                @ip1.copy!(@ip)
				@pts[0]=@ip1.position
            end
			view.invalidate
            view.tooltip="1st Corner of Pitch-Side: "+@ip.tooltip if @ip.valid?
        when 1 ### get the second point
            @ip.pick(view,x,y,@ip1)
            if @ip.valid? && @ip!=@ip2
                @ip2.copy!(@ip)
                @pts[1]=@ip2.position
            end
            view.invalidate
            view.tooltip="2nd Corner of Pitch-Side: "+@ip.tooltip if @ip.valid?
        when 2 ### get the third point
            @ip.pick(view,x,y,@ip2)
            if @ip.valid? && @ip!=@ip3
                @ip3.copy!(@ip)
                @pts[2]=@ip3.position
            end
            view.invalidate
            view.tooltip="Point on the Line of Other Side: "+@ip.tooltip if @ip.valid?
        end
    end
    def onLButtonDown(flags,x,y,view)
        @ip.pick(view,x,y)
        if @ip.valid?
            case @state
            when 0
                if @ip.valid?
                    @pts[0]=@ip.position
                    Sketchup::set_status_text("Roof Pyramid:  Pick the 2nd Corner of the Pitch-Side...")
                    @state=1
                end
            when 1
                if @ip.valid? && @ip!=@ip1
                    @pts[1]=@ip.position
                    @state=2
                    Sketchup::set_status_text("Roof Pyramid:  Pick a Point on the Line of the Other Side...")
                end
            when 2
                if @ip.valid? && @ip!=@ip1 && @ip!=@ip2
                    @pts[2]=@ip.position
                    @state=3
                    Roof::pyramid(@pts)
					return nil
                end
            end
        end
    end
    def onCancel(flag,view)
        view.invalidate if @drawn
        reset
    end
    def draw(view)
        if @ip.valid? && @ip.display?
            @ip.draw(view)
            @drawn=true
        end
        if @state==1
            view.set_color_from_line(@ip1,@ip)
            view.draw(GL_LINE_STRIP,@ip1.position,@ip.position)
            @drawn=true
        elsif @state==2
            view.drawing_color="orange"
            view.draw(GL_LINE_STRIP,@ip1.position,@ip2.position)
            view.set_color_from_line(@ip2,@ip)
            view.draw(GL_LINE_STRIP,@ip2.position,@ip.position)
            @drawn=true
        end
    end
    def onKeyDown(key,repeat,flags,view)
        if key==CONSTRAIN_MODIFIER_KEY && repeat==1
            @shift_down_time=Time.now
            if view.inference_locked?
                view.lock_inference
            elsif @ip.valid?
                view.lock_inference(@ip)
            end
        end
    end
    def onKeyUp(key,repeat,flags,view)
        if key==CONSTRAIN_MODIFIER_KEY && view.inference_locked? && Time.now-@shift_down_time>0.5
            view.lock_inference
        end
        if key==13
            Roof::pyramid(@pts)
            self.reset
        end
    end
end#class RoofPyramid
###---------------------------------------------------------------------


###---------------------------------------------------------------------

### Menus --------------------------------------------------------------
is_this_file=__FILE__
unless file_loaded?(is_this_file)
  menu=UI.menu("Plugins").add_submenu("Roof...")
  menu.add_item("Hipped Roof [from selected Face]"){Roof::hip}
  menu.add_item("Mansard | Sprocket Roof [from selected Face]"){Roof::mansard}
  menu.add_item("Gable-Ended Roof [pick 3 Points]"){Sketchup.active_model.select_tool RoofGable.new}
  menu.add_item("Pyramid Roof [pick 3 Points]"){Sketchup.active_model.select_tool RoofPyramid.new}
  menu.add_item("Get Slope [from selected Face]"){Roof::get_slope}
  menu.add_item("Help..."){Roof::help}
end#if
file_loaded(is_this_file)
###---------------------------------------------------------------------
